Package: SeqSQC
Title: A bioconductor package for sample quality check with next
        generation sequencing data
Version: 1.35.0
Authors@R: person("Qian", "Liu", email = "qliu7@buffalo.edu", role = c("aut", "cre"))
Description: The SeqSQC is designed to identify problematic samples in NGS data,
    including samples with gender mismatch, contamination, cryptic relatedness, and
    population outlier.
biocViews: Experiment Data, Homo_sapiens_Data, Sequencing Data,
        Project1000genomes, Genome
Depends: R (>= 3.4), ExperimentHub (>= 1.3.7), SNPRelate (>= 1.10.2)
License: GPL-3
Encoding: UTF-8
URL: https://github.com/Liubuntu/SeqSQC
BugReports: https://github.com/Liubuntu/SeqSQC/issues
RoxygenNote: 7.3.1
VignetteBuilder: knitr
Imports: e1071, GenomicRanges, gdsfmt, ggplot2, GGally, IRanges,
        methods, plotly, RColorBrewer, reshape2, rmarkdown, S4Vectors,
        stats, utils
Suggests: BiocStyle, knitr, testthat
git_url: https://git.bioconductor.org/packages/SeqSQC
git_branch: devel
git_last_commit: d08f659
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:06:15 UTC; biocbuild
Author: Qian Liu [aut, cre]
Maintainer: Qian Liu <qliu7@buffalo.edu>

# Generated by roxygen2: do not edit by hand

export(IBDCheck)
export(IBDRemove)
export(Inbreeding)
export(LoadVfile)
export(MissingRate)
export(PCACheck)
export(RenderReport)
export(SeqOpen)
export(SeqSQC)
export(SexCheck)
export(plotQC)
export(problemList)
export(sampleQC)
exportClasses(SeqSQC)
exportMethods("QCresult<-")
exportMethods("gdsfile<-")
exportMethods(QCresult)
exportMethods(gdsfile)
import(ExperimentHub)
import(GGally)
import(GenomicRanges)
import(IRanges)
import(RColorBrewer)
import(S4Vectors)
import(SNPRelate)
import(e1071)
import(gdsfmt)
import(ggplot2)
import(methods)
import(plotly)
import(reshape2)
importFrom(rmarkdown,render)
importFrom(stats,as.formula)
importFrom(stats,predict)
importFrom(utils,read.table)
importFrom(utils,write.table)

# SeqSQC
 SeqSQC is a bioconductor package for sample-level quality check in next generation sequencing (NGS) study. It is designed to automate and accelerate the sample cleaning of NGS data in any scale, including identifying problematic samples with high missing rate, gender mismatch, contamination, abnormal inbreeding coefficient, cryptic relatedness, and discordant population information. SeqSQC takes Variant Calling Format (VCF) files and sample annotation file containing sample population and gender information as input and report problematic samples to be removed from downstream analysis. Through incorporation a benchmark data assembled from the 1000 Genomes Project, it can accommodate NGS study of small sample size and low number of variants. 

#' Plot the QC results for specific QC steps. 
#' 
#' Plot QC results.
#' @param seqfile SeqSQC object with QC results.
#' @param QCstep which QC step the user want to do plotting. Takes
#'     values of \code{c("MissingRate", "SexCheck", "Inbreeding",
#'     "IBD", "PCA")}
#' @param interactive whether to generate interactive plot. Recommend
#'     to use \code{interactive = TRUE} if user perform sample QC
#'     using an rmarkdown script and output plot to html format.
#' @param sdcoef for inbreeding outlier check, how many standard
#'     deviation we need for identification of inbreeding
#'     outliers. The default is 5.
#' @param pc1 the eigenvector on x axis for PCA result. The default is
#'     "EV1" for eigenvector 1.
#' @param pc2 the eigenvector on y axis for PCA result. The default is
#'     "EV2" for eigenvector 2.
#' @param pairedScatter for PCA result, whether to plot the paired
#'     scatterplot for the first 4 PC axes.
#' @param ... Arguments to be passed to other methods.
#' @import ggplot2
#' @import RColorBrewer
#' @import GGally
#' @import plotly
#' @export
#' @return the ggplot or interactive plot (if output is in html
#'     format) for specific QC result. If "interactive=FALSE", it
#'     returns a ggplot and author could have the flexibility to add
#'     on any layers, scales, faceting specifications and coordinate
#'     systems.
#' @examples
#' load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
#' gfile <- system.file("extdata", "example.gds", package="SeqSQC")
#' seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
#' p <- plotQC(seqfile, QCstep="PCA", interactive=FALSE)
#' p
#' @author Qian Liu \email{qliu7@buffalo.edu}

plotQC <- function(seqfile,
                   QCstep=c("MissingRate", "SexCheck", "Inbreeding", "IBD", "PCA"),
                   interactive=FALSE, sdcoef=5, pc1="EV1", pc2="EV2",
                   pairedScatter=FALSE, ...){

    res.qc <- plotQCData(seqfile, QCstep = QCstep)

    ## plotting
    QCstep <- match.arg(QCstep)
    
    if(QCstep == "MissingRate"){
        p <- plotMissingRate(res.qc, interactive = interactive)
    }else if(QCstep == "SexCheck"){
        p <- plotSexCheck(res.qc, interactive = interactive)
    }else if(QCstep == "Inbreeding"){
        p <- plotInbreeding(res.qc, interactive = interactive, sdcoef = sdcoef)
    }else if(QCstep == "IBD"){
        p <- plotIBD(res.qc, interactive = interactive)
    }else if(QCstep == "PCA"){
     p <- plotPop(res.qc, interactive = interactive, ...)
    }
    return(p)
}

subsetGDS <- function(gds, output.gds=NULL, sample.idx=NULL, snp.idx=NULL){

    if (!inherits(gds, "gds.class")){
        return("object should inherit from 'gds.class'.")
    }

    ## when both "sample.idx" and "snp.idx" is defined, we remove snps first, and then remove samples, and then remove corresponding homozygous ref sites. 

    filepath <- gds$filename
    closefn.gds(gds)
    rm(gds)
    ## cleanup.gds(filepath)
    
    ## write to a new gds file. 
    if (!is.null(output.gds)){
        file.copy(filepath, output.gds, overwrite=TRUE)
        gds <- openfn.gds(output.gds, readonly=FALSE)
    }else{
        gds <- openfn.gds(filepath, readonly=FALSE)
    }
    
    if(!("gds.class" %in% class(gds)))stop("Not gds format!")
    nodes <- ls.gdsn(gds)
    snp.nodes <- nodes[grep("snp", nodes)]

    ## GDS subset for SNPs.
    if(!is.null(snp.idx)){
        for(i in snp.nodes){
            if(i=="snp.annot"){
                sa <- index.gdsn(gds, "snp.annot")
                for(j in ls.gdsn(sa)){
                    assign.gdsn(index.gdsn(sa, j), seldim = snp.idx)
                }
            }else {
                assign.gdsn(index.gdsn(gds, i), seldim = snp.idx)
            }
        }
        assign.gdsn(index.gdsn(gds, "genotype"), seldim = list(snp.idx, NULL))
    }
    
    ## GDS subset for samples, (do not check homozygous reference SNPs after removing samples.)
    sample.id <- index.gdsn(gds, "sample.id")
    if("sample.annot" %in% ls.gdsn(gds$root)){
        sample.annot <- index.gdsn(gds, "sample.annot")
    }else{
        sample.annot <- NULL
    }

    if(!is.null(sample.idx)){
        assign.gdsn(sample.id, seldim = sample.idx)
        assign.gdsn(index.gdsn(gds, "genotype"), seldim = list(NULL, sample.idx))
        if(!is.null(sample.annot)){
            for(i in ls.gdsn(sample.annot)){
                assign.gdsn(index.gdsn(sample.annot, i), seldim = sample.idx)
            }
        }
        
        ## ## check if any SNPs does not have variants after removing samples, and remove those SNPs.
        ## ## snp.idx.nhr <- unlist(apply.gdsn(index.gdsn(gds, "genotype"), 1, function(x) sum(x) > 0)) ## not correct, 0/1/2 is dosage of ref alleles, should count 2 for homo-ref here. 
        ## nsamples <- length(read.gdsn(sample.id))
        ## ## snp.idx.nhr <- unlist(apply.gdsn(index.gdsn(gds, "genotype"), 1, function(x) sum(x) < nsamples*2))
        ## snp.idx.nhr <- unlist(apply.gdsn(index.gdsn(gds, "genotype"), 1, function(x) sum(x==2) < nsamples))
        ## ## take ~20 min for whole genome seq data.
        
        ## if(any(!snp.idx.nhr)){
        ##     for(i in snp.nodes){
        ##         if(i=="snp.annot"){
        ##             sa <- index.gdsn(gds, "snp.annot")
        ##             for(j in ls.gdsn(sa)){
        ##                 assign.gdsn(index.gdsn(sa, j), seldim = snp.idx.nhr)
        ##             }
        ##         }else {
        ##             assign.gdsn(index.gdsn(gds, i), seldim = snp.idx.nhr)
        ##         }
        ##     }
        ##     assign.gdsn(index.gdsn(gds, "genotype"), seldim = list(snp.idx.nhr, NULL))
        ## }
    }

    closefn.gds(gds)
    if (!is.null(output.gds)){
        cleanup.gds(output.gds)
    }
    else{
        cleanup.gds(filepath)
    }
}

#' Data preprocessing for VCF or plink input from NGS or GWAS data.
#' 
#' Function to read VCF or plink files, merge with benchmark data, and
#' output as SeqSQC object.
#' @param vfile vcf or PLINK input file (ped/map/bed/bim/fam with same
#'     basename). Vfile could be a vector of character strings, see
#'     details.
#' @param output a character string for name of merged data of SeqSQC
#'     object. The \code{dirname(output)} would be used as the
#'     directory to save the QC results and plots. The default is
#'     \code{sampleqc} in working directory.
#' @param capture.region the BED file of sequencing capture
#'     regions. The default is NULL. For exome-sequencing data, the
#'     capture region file must be provided.
#' @param sample.annot sample annotation file with 3 columns (with
#'     header) in the order of sample id, sample population and sex
#'     info. The default is NULL.
#' @param LDprune whether to use LD-pruned snp set. The default is
#'     TRUE.
#' @param vfile.restrict whether the input vcf or plink file has
#'     already been restricted by capture region. The default is
#'     FALSE.
#' @param slide.max.bp the window size of SNPs when calculating
#'     linkage disequilibrium. The default is 5e+05.
#' @param ld.threshold the r^2 threshold for LD-based SNP pruning if
#'     \code{LDprune = TRUE}. The default is 0.3.
#' @param format.data the data source. The default is \code{NGS} for
#'     sequencing data.
#' @param format.file the data format. The default is \code{vcf}.
#' @param ... Arguments to be passed to other methods.
#' @export
#' @return a SeqSQC object with the filepath to the gds file which
#'     stores the genotype, the summary of samples and variants, and
#'     the QCresults including the sample annotation information.
#' @details For \code{vfile} with more than one file names,
#'     \code{LoadVfile} will merge all dataset together if they all
#'     contain the same samples. It is useful to combine
#'     genetic/genomic data together if VCF data is divided by
#'     chromosomes. \cr \code{sample.annot} file contains 3 columns
#'     with column names. col 1 is \code{sample} with sample ids; col
#'     2 is \code{population} with values of "AFR/EUR/ASN/EAS/SAS";
#'     col 3 is \code{gender} with values of "male/female".
#' @import gdsfmt
#' @import SNPRelate
#' @import GenomicRanges
#' @import IRanges
#' @import S4Vectors
#' @import methods
#' @import ExperimentHub
#' @importFrom utils read.table
#' @examples
#' infile <- system.file("extdata", "example_sub.vcf", package="SeqSQC")
#' sample.annot <- system.file("extdata", "sampleAnnotation.txt", package="SeqSQC")
#' cr <- system.file("extdata", "CCDS.Hs37.3.reduced_chr1.bed", package="SeqSQC")
#' outfile <- file.path(tempdir(), "testWrapUp")
#' seqfile <- LoadVfile(vfile = infile, output = outfile, capture.region = cr,
#' sample.annot = sample.annot)
#' @author Qian Liu \email{qliu7@@buffalo.edu}

LoadVfile <- function(vfile, output="sampleqc", capture.region=NULL,
                      sample.annot=NULL, LDprune=TRUE,
                      vfile.restrict=FALSE, slide.max.bp=5e+05,
                      ld.threshold=0.3, format.data="NGS",
                      format.file="vcf", ...){

    tmpdir <- tempdir()
    study.gds <- tempfile(tmpdir=tmpdir)
    ## 1. read in vcf/plink file from study cohort, convert to gds format. 
    if(format.file == "vcf"){
        message("Load vcf file ...")
        ## study.gds <- "study.gds"
        snpgdsVCF2GDS(vfile, study.gds, method="biallelic.only",
                      snpfirstdim=TRUE, ...)
    }else if(format.file == "plink"){
        message("Load plink file ...")
        ## study.gds <- "study.gds"
        vfile.base <- sub("ped|map|bed|bim|fam", "", vfile)
        if(length(grep("ped|map", vfile)) != 0){
            vfile.ped <- paste0(vfile.base, "ped")
            vfile.map <- paste0(vfile.base, "map")
            snpgdsPED2GDS(vfile.ped, vfile.map, study.gds,
                          family=TRUE, snpfirstdim=TRUE, ...)
        }else{
            vfile.bed <- paste0(vfile.base, "bed")
            vfile.fam <- paste0(vfile.base, "fam")
            vfile.bim <- paste0(vfile.base, "bim")
            snpgdsBED2GDS(vfile.bed, vfile.fam, vfile.bim, study.gds,
                          family=TRUE, snpfirstdim=TRUE, ...)
        }
    }else{
        stop("wrong file type")
    }
    ## vtmp <- tempfile(tmpdir=tmpdir)
    ## file.copy(study.gds, vtmp)
    studycohort <- openfn.gds(study.gds, readonly=FALSE) 
    samples <- read.gdsn(index.gdsn(studycohort, "sample.id"))
    
    ## 2. read in the annotation for the study samples.
    if(!"sample.annot" %in% ls.gdsn(studycohort)){
        message("Load study cohort annotation file ...")
        study.annot <- read.table(file=sample.annot, as.is=TRUE,
                                  header=TRUE, stringsAsFactors=FALSE)
        names(study.annot) <- c("sample", "population", "gender")
        study.annot$relation <- NA
        study.annot$group <- "study"
        study.annot <- study.annot[match(samples, study.annot$sample), ]
        add.gdsn(studycohort, "sample.annot", val=study.annot, check=FALSE)
    }else{
        message("Rewrite study cohort annotation file for PLINK data ...")
        study.annot <- read.table(file=sample.annot, as.is=TRUE,
                                  header=TRUE, stringsAsFactors=FALSE)
        sampleanno <- read.gdsn(index.gdsn(studycohort, "sample.annot"))

        relations <- rep("", length(samples))
        ind1 <- sampleanno[, "father"] != 0 & sampleanno[, "mother"] != 0
        relations[ind1] <- paste0("father:",
                                  sampleanno[ind1, "father"],
                                  ";mother:",
                                  sampleanno[ind1, "mother"])
        ind2 <- sampleanno[, "father"] != 0 & sampleanno[, "mother"] == 0
        relations[ind2] <- paste0("father:", sampleanno[ind2, "father"])
        ind3 <- sampleanno[, "father"] == 0 & sampleanno[, "mother"] != 0
        relations[ind3] <- paste0("mother:", sampleanno[ind3, "mother"])

        ind <- match(samples, study.annot$sample)
        annot <- data.frame(sample=samples,
                            population=study.annot[ind, "population"],
                            gender=study.annot[ind, "gender"],
                            relation=relations, group="study",
                            stringsAsFactors=FALSE)
        add.gdsn(studycohort, "sample.annot", val=annot, check=FALSE, replace=TRUE)
    }        

    ## 3. read in benchmark gds file for 87 samples from 1000 genomes project.
    message("Load 1kg data to temp directory...")
    ## gds.1kg <- system.file("extdata", "benchmark_1000genomes.gds", package="SeqSQC")
    dfile <- ExperimentHub()[["EH550"]]
    gds.1kg <- dfile$filename
    closefn.gds(dfile)
    ## gds.1kg <- fileName(ExperimentHub())[["EH550"]]
    tmp.1kg <- tempfile(tmpdir=tmpdir)
    file.copy(gds.1kg, tmp.1kg)
    genofile <- openfn.gds(tmp.1kg, readonly = FALSE)

    ## 4. capture benchmark data (and/or) study data with Capture region
    if(!is.null(capture.region)){
        cp <- read.table(capture.region, sep="\t", stringsAsFactors=FALSE)
        cp <- GRanges(cp[,1], IRanges(as.numeric(cp[,2])+1, as.numeric(cp[,3])))
        
        chr.1kg <- read.gdsn(index.gdsn(genofile, "snp.chromosome"))
        pos.1kg <- read.gdsn(index.gdsn(genofile, "snp.position"))
        gr.1kg <- GRanges(chr.1kg, IRanges(pos.1kg, pos.1kg))
        ov <- findOverlaps(gr.1kg, cp)
        message("Subset 1kg data to capture region...")
        subsetGDS(genofile, snp.idx=unique(queryHits(ov)))
        genofile <- openfn.gds(tmp.1kg, readonly=TRUE)
    
        if(vfile.restrict==FALSE){
            chr.sc <- read.gdsn(index.gdsn(studycohort, "snp.chromosome"))
            pos.sc <- read.gdsn(index.gdsn(studycohort, "snp.position"))
            gr.sc <- GRanges(chr.sc, IRanges(pos.sc, pos.sc))
            message("restrict Vfile by capture region...")
            ov <- findOverlaps(gr.sc, cp)
            subsetGDS(studycohort, snp.idx = unique(queryHits(ov)))
            cleanup.gds(study.gds)
            studycohort <- openfn.gds(study.gds, readonly=TRUE)
        }
    }
    
    ## 5. merge study cohort with benchmark data
    message("Merging gds files of 1kg data and study cohort ...")
    output.merge <- paste0(output, ".gds")

    if(format.data == "NGS"){
        merge.out <- mergeGDS(gds1=genofile, gds2=studycohort,
                              output=output.merge, missing.fill=TRUE)
        message("keep only snps within benchmark...")
        snp.bm <- paste(read.gdsn(index.gdsn(genofile, "snp.chromosome")),
                        read.gdsn(index.gdsn(genofile, "snp.position")),
                        read.gdsn(index.gdsn(genofile, "snp.allele")),
                        sep="_")
        snp.merge <- paste(read.gdsn(index.gdsn(merge.out, "snp.chromosome")),
                           read.gdsn(index.gdsn(merge.out, "snp.position")),
                           read.gdsn(index.gdsn(merge.out, "snp.allele")),
                           sep="_")
        bmtf <- snp.merge %in% snp.bm
        subsetGDS(gds=merge.out, snp.idx = bmtf)
        merge.out <- openfn.gds(output.merge, readonly=FALSE)
    }else{
        merge.out <- mergeGDS(gds1=genofile, gds2=studycohort,
                              output=output.merge, missing.fill=FALSE)
    }
    class(merge.out) <- c("SNPGDSFileClass", "gds.class")
    
    ## LD pruning
    if(LDprune){
        message("LD pruning ...")
        ldprunein <- snpgdsLDpruning(merge.out,
                                     slide.max.bp=slide.max.bp,
                                     ld.threshold=ld.threshold)
        ldtf <- read.gdsn(index.gdsn(merge.out, "snp.id")) %in% unlist(ldprunein)
        if(! "snp.annot" %in% ls.gdsn(merge.out)){
            af <- addfolder.gdsn(merge.out, "snp.annot")
        }else{
            af <- index.gdsn(merge.out, "snp.annot")
        }
        add.gdsn(af, "LDprune", storage="logical", ldtf)
    }

    closefn.gds(genofile)
    closefn.gds(studycohort)
    unlink(tmp.1kg)
    unlink(study.gds)
    
    fn <- merge.out$filename

    nds <- c("sample.id", "sample.annot", "snp.id") 
    allnds <- lapply(nds, function(x) read.gdsn(index.gdsn(merge.out, x)))
    names(allnds) <- c("samples", "sampleanno", "snps")
    
    ## sampleanno <- read.gdsn(index.gdsn(merge.out, "sample.annot")) 
    closefn.gds(merge.out)

    output <- SeqSQC(gdsfile = fn,
                     QCresult = SimpleList(dimension = c(length(allnds$samples), length(allnds$snps)),
                                           sample.annot = allnds$sampleanno))
    return(output)
}

.onLoad <- function(libname = find.package("SeqSQC"), pkgname = "SeqSQC"){

    if(getRversion() >= "2.15.1"){
        utils::globalVariables(c("PC1", "PC2", "id1", "id2",
                                 "inbreeding", "k0", "k1", "label",
                                 "missingRate", "pop", "pred.label",
                                 "pred.pop", "pred.sex",
                                 "problematic", "relation", "sex",
                                 "sexinb", "type"))
    }
    invisible()
}

plotMissingRate <- function(res.qc, interactive = FALSE){

    res.pop.ord <- res.qc
    res.pop.ord$outlier <- ifelse(res.pop.ord$missingRate > 0.1, "Yes", "No")
    res.pop.ord$problematic <- factor(res.pop.ord$outlier, levels=c("Yes", "No"))
    
    p <- ggplot(data=res.pop.ord, aes(x=sample, y=missingRate))
    p <- p + geom_point(aes(colour=problematic, shape=type))
    p <- p + scale_color_discrete(drop = FALSE)
    p <- p + geom_hline(yintercept=0.1, colour="grey")
    p <- p + theme(axis.text.x=element_blank())

    if(interactive){
        return(ggplotly(p))
    }else{
        return(p)
    }

}

mergeGDS <- function(gds1, gds2, output, missing.fill=TRUE){
    if(!("gds.class" %in% class(gds1)))stop("Not gds.class!")
    if(!("gds.class" %in% class(gds2)))stop("Not gds.class!")
    ## merge samples
        
    gds.out <- createfn.gds(output, allow.duplicate = TRUE)

    message("merging sample information...")
    ids <- c(read.gdsn(index.gdsn(gds1, "sample.id")), read.gdsn(index.gdsn(gds2, "sample.id")))
    anno <- rbind(read.gdsn(index.gdsn(gds1, "sample.annot")), read.gdsn(index.gdsn(gds2, "sample.annot")))
    add.gdsn(gds.out, "sample.id", ids)
    add.gdsn(gds.out, "sample.annot", anno, check = FALSE)
    message("merging sample information... done!")
    
    ## merge snps
    message("merging snp information...")

    snpn <- c("snp.id", "snp.rs.id", "snp.chromosome", "snp.position", "snp.allele") 
    snpnodes1 <- lapply(
        snpn,
        function(x){
            read.gdsn(index.gdsn(gds1, x))
        }
    )
    snpnodes2 <- lapply(
        snpn,
        function(x){
            read.gdsn(index.gdsn(gds2, x))
        }
    )
    names(snpnodes1) <- names(snpnodes2) <- snpn

    snp1 <- paste(
        snpnodes1$snp.chromosome,
        snpnodes1$snp.position,
        snpnodes1$snp.allele,
        sep="_")
    snp2 <- paste(
        snpnodes2$snp.chromosome,
        snpnodes2$snp.position,
        snpnodes2$snp.allele,
        sep="_")
    
    if(missing.fill == TRUE){
        ## fill non-intersect genotypes as homozygous reference.
        d21 <- !(snp2 %in% snp1)
        d12 <- !(snp1 %in% snp2)
        snps1 <- c(snp1, snp2[d21])
        snps2 <- c(snp2, snp1[d12])
        
        add.gdsn(
            gds.out,
            "snp.id",
            c(snpnodes1$snp.id, max(snpnodes1$snp.id) + snpnodes2$snp.id[d21])
        )
        add.gdsn(gds.out,
                 "snp.rs.id",
                 c(snpnodes1$snp.rs.id, snpnodes2$snp.rs.id[d21])
                 )
        add.gdsn(
            gds.out,
            "snp.chromosome",
            c(snpnodes1$snp.chromosome, snpnodes2$snp.chromosome[d21])
        )
        add.gdsn(
            gds.out,
            "snp.position",
            c(snpnodes1$snp.position, snpnodes2$snp.position[d21])
        )
        add.gdsn(
            gds.out,
            "snp.allele",
            c(snpnodes1$snp.allele, snpnodes2$snp.allele[d21])
        )
        message("merging snp information... done!")
        
        message("merging genotypes...")
        add.gdsn(gds.out, "genotype", storage = "bit2", valdim=c(length(snps1), 0))

        apply.gdsn(index.gdsn(gds1, "genotype"), 2, function(x){
            append.gdsn(index.gdsn(gds.out, "genotype"), c(x, rep(2, sum(d21)))) ## fill as homo ref.
        })
        apply.gdsn(index.gdsn(gds2, "genotype"), 2, function(x){
            append.gdsn(index.gdsn(gds.out, "genotype"), c(x, rep(2, sum(d12)))[match(snps1, snps2)])
        })
        message("mergeing genotypes... done!")
        
        if("snp.annot" %in% ls.gdsn(gds1) & "snp.annot" %in% ls.gdsn(gds2)){
            message("merging snp annotation file for NGS data ...")
            
            snpanno <- c("snp.annot/qual", "snp.annot/filter")
            snpanno1 <- lapply(
                snpanno,
                function(x){
                    read.gdsn(index.gdsn(gds1, x))
                }
            )
            snpanno2 <- lapply(
                snpanno,
                function(x){
                    read.gdsn(index.gdsn(gds2, x))
                }
            )
            names(snpanno1) <- names(snpanno2) <- c("snp.qual", "snp.filter")
            
            af <- addfolder.gdsn(gds.out, "snp.annot")
            add.gdsn(af, "qual", c(snpanno1$snp.qual, snpanno2$snp.qual[d21]))
            add.gdsn(af, "filter", c(snpanno1$snp.filter, snpanno2$snp.filter[d21]))
            message("merging snp annotation file for NGS data ... done!")
        }
        
    }else{
        ## only keep the intersection snps.
        ind <- snp1 %in% snp2
        add.gdsn(gds.out, "snp.id", snpnodes1$snp.id[ind])
        add.gdsn(gds.out, "snp.rs.id", snpnodes1$snp.rs.id[ind])
        add.gdsn(gds.out, "snp.chromosome", snpnodes1$snp.chromosome[ind])
        add.gdsn(gds.out, "snp.position", snpnodes1$snp.position[ind])
        add.gdsn(gds.out, "snp.allele", snpnodes1$snp.allele[ind])
        message("merging snp information... done!")
        
        message("merging genotypes...")
        add.gdsn(gds.out, "genotype", storage = "bit2", valdim=c(sum(ind), 0))
        ind2 <- match(snp1, snp2)
        ind22 <- ind2[!is.na(ind2)]
        apply.gdsn(index.gdsn(gds1, "genotype"), 2, function(x){
            append.gdsn(index.gdsn(gds.out, "genotype"), x[ind])
        })
        apply.gdsn(index.gdsn(gds2, "genotype"), 2, function(x){
            append.gdsn(index.gdsn(gds.out, "genotype"), x[ind22])
        })
        message("mergeing genotypes... done!")
        ## message("filtering variants with MAF<0.05...")
        ## mr <- apply.gdsn(index.gdsn(gds.out, "genotype"), 1, function(x){  
        ##     1 - sum(x %in% c(0, 1, 2))/length(x)
        ## })
    }
    return(gds.out)
}

#' Render the rmarkdown file for generating a sample QC report.
#' 
#' Function to render the pre-compiled rmarkdown file to generate the
#' sample QC report.
#' @param input SeqSQC object with QC results.
#' @param output a character string to define the file name for the QC
#'     report.
#' @param interactive whether to generate interative plots in the
#'     report. The default is TRUE.
#' @importFrom rmarkdown render
#' @return Will incure the rendering of the rmarkdown file for
#'     generating the sample QC report. The report will return to the
#'     file denoted in \code{output} in the function.
#' @export
#' @author Qian Liu \email{qliu7@@buffalo.edu}
#' @examples
#' load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
#' RenderReport(seqfile, output="report.html", interactive=FALSE)

RenderReport <- function(input, output, interactive = TRUE){
    report <- system.file("extdata", "sampleQCReport.Rmd", package="SeqSQC")
    render(report,
           output_file = basename(output),
           output_dir=dirname(output),
           params=list(
               seqfile = input,
               interactive = interactive
           ))
}

#' Sample relationship check with SeqSQC object input file. 
#'
#' Function to calculate the IBD coefficients for all sample pairs and
#' to predict related sample pairs in study cohort.
#'
#' @param seqfile SeqSQC object, which includes the merged gds file
#'     for study cohort and benchmark.
#' @param remove.samples a vector of sample names for removal from IBD
#'     calculation. Could be problematic samples identified from
#'     previous QC steps, or user-defined samples.
#' @param LDprune whether to use LD-pruned snp set. The default is
#'     TRUE.
#' @param kin.filter whether to use "kinship coefficient >= 0.08" as
#'     the additional criteria for related samples. The default is
#'     TRUE.
#' @param missing.rate to use the SNPs with "<= \code{missing.rate}"
#'     only; if NaN, no threshold. By default, we use
#'     \code{missing.rate = 0.1} to filter out variants with missing
#'     rate greater than 10\%.
#' @param ss.cutoff the minimum sample size (300 by default) to apply
#'     the MAF filter. This sample size is the sum of study samples
#'     and the benchmark samples of the same population as the study
#'     cohort.
#' @param maf to use the SNPs with ">= \code{maf}" if sample size
#'     defined in above argument is greater than \code{ss.cutoff};
#'     otherwise NaN is used by default for no MAF threshold.
#' @param hwe to use the SNPs with Hardy-Weinberg equilibrium p >=
#'     \code{hwe} if sample size defined in above argument is greater
#'     than \code{ss.cutoff}; otherwise no hwe threshold. The default
#'     is 1e-6.
#' @param ... Arguments to be passed to other methods.
#' @keywords IBD
#' @return a data frame with sample names, the descent coefficients of
#'     k0, k1 and kinship, self-reported relationship and predicted
#'     relationship for each pair of samples.
#' @details Using LD-pruned variants (by default), we calculate the
#'     IBD coefficients for all sample pairs, and then predict related
#'     sample pairs in study cohort using the support vector machine
#'     (SVM) method with linear kernel and the known relatedness
#'     embedded in benchmark data as training set. \cr Sample pairs
#'     with discordant self-reported and predicted relationship are
#'     considered as problematic. All predicted related pairs are also
#'     required to have coefficient of kinship >= 0.08 by default. The
#'     sample with higher missing rate in each related pair is
#'     selected for removal from further analysis by function of
#'     \code{IBDRemove}.
#' @import RColorBrewer
#' @import e1071
#' @import reshape2
#' @export
#' @examples
#' load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
#' gfile <- system.file("extdata", "example.gds", package="SeqSQC")
#' seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
#' seqfile <- IBDCheck(seqfile, remove.samples=NULL, LDprune=TRUE, missing.rate=0.1)
#' res.ibd <- QCresult(seqfile)$IBD
#' tail(res.ibd)
#' @author Qian Liu \email{qliu7@@buffalo.edu}


IBDCheck <- function(seqfile, remove.samples = NULL, LDprune = TRUE,
                     kin.filter = TRUE, missing.rate = 0.1,
                     ss.cutoff = 300, maf = 0.01, hwe = 1e-6, ...){

    ## check
    if (!inherits(seqfile, "SeqSQC")){
        return("object should inherit from 'SeqSQC'.")
    }

    message("calculating pairwise IBD ...")
    
    gfile <- SeqOpen(seqfile, readonly=TRUE)
    
    nds <- c("sample.id", "sample.annot", "snp.id") 
    allnds <- lapply(nds, function(x) read.gdsn(index.gdsn(gfile, x)))
    names(allnds) <- c("samples", "sampleanno", "snp.id")
    
    studyid <- allnds$sampleanno[allnds$sampleanno[,5] == "study", 1]
    
    ## sample filters. (remove prespecified "remove.samples", and only
    ## keep samples within the study population and benchmark data)
    study.pop <- unique(allnds$sampleanno[allnds$sampleanno$group == "study", "population"])
    if (length(study.pop) > 1)
        stop("Study samples should be single population, please prepare input file accordingly.")
    
    if (!is.null(remove.samples)){
        flag <- (allnds$sampleanno$group != "study" | allnds$sampleanno$population == study.pop) &
            !allnds$samples %in% remove.samples
    } else {
        flag <- allnds$sampleanno$group != "study" | allnds$sampleanno$population == study.pop
    }
    sample.ibd <- allnds$samples[flag]
    
    ## add hwe filter for variants when sample size >= 300.
    if (length(sample.ibd) >= ss.cutoff){
        snp.hwe <- snpgdsHWE(gfile, sample.id=sample.ibd)
        hwe.idx <- snp.hwe > hwe & !is.na(snp.hwe)
    } else {
        hwe.idx <- rep(TRUE, length(allnds$snp.id))
    }
    
    ## use LDpruned SNPs if LDprune == TRUE.
    if(LDprune){
        ld <- read.gdsn(index.gdsn(gfile, "snp.annot/LDprune"))
    }else{
        ld <- rep(TRUE, length(allnds$snp.id))
    }
    
    ## SNP filters in together. (HWE+LDprune)
    snp.idx <- ld & hwe.idx
    
    ## use maf filter if sample size >= 300.
    if (length(sample.ibd) >= ss.cutoff){ 
        IBD.res <- snpgdsIBDMoM(gfile, sample.id=sample.ibd,
                                snp.id=allnds$snp.id[snp.idx],
                                kinship=TRUE, maf=maf,
                                missing.rate=missing.rate, ...)
    }else{
        IBD.res <- snpgdsIBDMoM(gfile, sample.id=sample.ibd,
                                snp.id=allnds$snp.id[snp.idx],
                                kinship=TRUE, maf=NaN,
                                missing.rate=missing.rate, ...)
    }
    closefn.gds(gfile)

    k0 <- IBD.res$k0
    k1 <- IBD.res$k1
    kin <- IBD.res$kinship
    
    ## We generate a big dataset for pairwise IBD.res result.
    sample.id.ibd <- IBD.res$sample.id
    rownames(kin) <- rownames(k1) <- rownames(k0) <- sample.id.ibd
    colnames(kin) <- colnames(k1) <- colnames(k0) <- sample.id.ibd
    idx <- melt(upper.tri(k0))[,3]
    k0m <- melt(k0)[idx,]
    k1m <- melt(k1)[idx,]
    kinm <- melt(kin)[idx,]
    res.ibd <- cbind(k0m, k1m[,3], kinm[,3])
    colnames(res.ibd) <- c("id1", "id2", "k0", "k1", "kin")        
    res.ibd$id1 <- as.character(res.ibd$id1)
    res.ibd$id2 <- as.character(res.ibd$id2)
    
    ## 1000g benchmark sample relation
    PO <- rbind(c("NA19238", "NA19240"),
                c("NA19239", "NA19240"),
                c("NA20868", "NA20871"),
                c("NA20886", "NA20898"))
    FS <- rbind(c("HG00581", "HG00635"),
                c("NA19713", "NA19985"))
    HF <- rbind(c("HG00119", "HG00124"),
                c("HG02353", "HG02363")
                )
    
    res.ibd$label <- "UN"
    res.ibd$label[paste(res.ibd$id1, res.ibd$id2) %in% paste(PO[,1], PO[,2])] <- "PO"
    res.ibd$label[paste(res.ibd$id1, res.ibd$id2) %in% paste(FS[,1], FS[,2])] <- "FS"
    res.ibd$label[paste(res.ibd$id1, res.ibd$id2) %in% paste(HF[,1], HF[,2])] <- "HF"
    res.ibd$label <- factor(res.ibd$label)
    
    ## index for 1000g benchmark data
    idx <- !(res.ibd$id1 %in% studyid | res.ibd$id2 %in% studyid)
    
    ## define relation centers
    centers <- data.frame(k0 = c(0, 0.25, 0.5, 1),
                          k1 = c(1, 0.5, 0.5, 0),
                          label = c("PO", "FS", "HF", "UN"),
                          stringsAsFactors=FALSE)
    
    ## if(method.pred=="svm"){
    ## use 1000g benchmark related samples and centers as training data. 
    ## svm: descriminate PO, FS, HF, FC, UN.
    res.svm <- rbind(res.ibd[idx, c("k0", "k1", "label")], centers[, c("k0", "k1", "label")])
    res.svm$label <- factor(res.svm$label)
    model <- svm(label ~ k0 + k1, data=res.svm, probability = FALSE, kernel="linear")
    ## predict the study sample relations, with probabilities.
    pred.svm <- predict(model, res.ibd[, c("k0", "k1")], probability = FALSE)
    pred.svm <- factor(pred.svm, levels=c("DU", levels(pred.svm)))
    pred.svm[res.ibd$kin > 0.46] <- "DU"
    ## use kin > 0.08 as additional filter.
    if(kin.filter){
        pred.svm[pred.svm != "UN" & res.ibd$kin < 0.08] <- "UN"
    }
    res.ibd <- data.frame(res.ibd[, 1:6], pred.label = pred.svm, stringsAsFactors=FALSE)
    res.ibd$pred.label <- as.character(res.ibd$pred.label)
    res.ibd$pred.label[res.ibd$pred.label != "UN"] <- "Related"
    res.ibd$pred.label <- factor(res.ibd$pred.label)
    ## }

    ## fully executable, could resume when updating.
    ## if(method.pred == "vglm"){
    ##     ## library(VGAM) ## importFrom VGAM vglm
    ##     ## use 1000g benchmark related samples and centers as training data.
    ##     ## vglm: descriminate PO, FS, HF, FC, UN.
    ##     res.vglm <- rbind(res.ibd[idx, c("k0", "k1", "label")], centers[, c("k0", "k1", "label")])
    ##     res.vglm$label <- factor(res.vglm$label)
    ##     model <- vglm(label ~ k0 + k1, data=res.vglm, family = multinomial)
    ##     probabilities <- predict(model, res.ibd[, c("k0", "k1")], type="response")
    ##     pred.vglm <- apply(probabilities, 1, which.max)
    ##     pred.vglm[which(pred.vglm=="1")] <- levels(res.vglm$label)[1]
    ##     pred.vglm[which(pred.vglm=="2")] <- levels(res.vglm$label)[2]
    ##     pred.vglm[which(pred.vglm=="3")] <- levels(res.vglm$label)[3]
    ##     pred.vglm[which(pred.vglm=="4")] <- levels(res.vglm$label)[4]

    ##     ## identify duplicates
    ##     pred.vglm[res.ibd$kin > 0.46] <- "DU"
    ##     pred.vglm <- factor(pred.vglm)
    ##     ## use kin > 0.08 as additional filter.
    ##     if(kin.filter){
    ##         pred.vglm[pred.vglm != "UN" & res.ibd$kin < 0.08] <- "UN"
    ##     }
    ##     res.ibd <- data.frame(res.ibd[, 1:6], pred.label = pred.vglm)
    ##     res.ibd$pred.label <- as.character(res.ibd$pred.label)
    ##     res.ibd$pred.label[res.ibd$pred.label != "UN"] <- "Related"
    ##     res.ibd$pred.label <- factor(res.ibd$pred.label)
    ## }
    
    ## return the SeqSQC object with updated QC results.
    a <- QCresult(seqfile)
    a$IBD <- res.ibd
    outfile <- SeqSQC(gdsfile = gdsfile(seqfile), QCresult = a)
    return(outfile)


}

plotPop <- function(res.qc, pc1="EV1", pc2="EV2", interactive=FALSE, pairedScatter=FALSE){
    if(pairedScatter){
        p <- ggpairs(
            res.qc,
            columns=c("EV1", "EV2", "EV3", "EV4"),
            aes(colour=pop, shape=type),
            upper="blank"
        )
    }else{
        cols <- match(c(pc1, pc2), colnames(res.qc))
        res.qc2 <- res.qc[, c(1:3, cols, 8)]
        colnames(res.qc2)[4:5] <- c("PC1", "PC2")
        p <- ggplot(data=res.qc2, aes(x=PC1, y=PC2))
        p <- p + geom_point(aes(colour=pop, shape=type))
        prob.pca <- res.qc2[res.qc2$pop != res.qc2$pred.pop, ]
        if(nrow(prob.pca) > 0){
            p <- p +
                geom_point(
                    data=prob.pca,
                    aes(x=PC1, y=PC2), shape=1, colour="red", size=4
                )
        }
        p <- p + theme_classic()
        ## p <- p + theme(legend.position = c(0.7, 0.8))
        p <- p + labs(x="Principle component 1",
                      y="Principle component 2",
                      col="Population",
                      shape="Type")
    }

    if(interactive){
        return(ggplotly(p))
    }else{
        return(p)
    }
}

#' The wrap-up function for sample QC of sequencing/GWAS data.
#' 
#' A wrap-up function for sample QC. It reads in the variant genotypes
#' in vcf/PLINK format, merges study cohort with benchmark data, and
#' performs sample QC for the merged dataset.
#' 
#' @param vfile vcf or PLINK input file (ped/map/bed/bim/fam with same
#'     basename). The default is NULL. Vfile could be a vector of
#'     character strings, see details. Could also take file in
#'     \code{SeqSQC} object generated from \code{LoadVfile}.
#' @param output a character string for name of merged data of SeqSQC
#'     object. The \code{dirname(output)} would be used as the
#'     directory to save the QC result and plots. The default is
#'     \code{sampleqc} in the working directory.
#' @param capture.region the BED file of sequencing capture
#'     regions. The default is NULL. For exome-sequencing data, the
#'     capture region file must be provided.
#' @param sample.annot sample annotation file with 3 columns (with
#'     header) in the order of sample id, sample population and sex
#'     info. The default is NULL.
#' @param LDprune whether to use LD-pruned snp set. The default is
#'     TRUE.
#' @param vfile.restrict whether the input vcf or plink file has
#'     already been restricted by capture region. The default is
#'     FALSE.
#' @param slide.max.bp the window size of SNPs when calculating
#'     linkage disequilibrium. The default is 5e+05.
#' @param ld.threshold the r^2 threshold for LD-based SNP pruning if
#'     \code{LDprune = TRUE}. The default is 0.3.
#' @param format.data the data source. The default is \code{NGS} for
#'     sequencing data.
#' @param format.file the data format. The default is \code{vcf}.
#' @param QCreport Whether to generate the sample QC report in html
#'     format.
#' @param out.report the file name for the sample QC report. The
#'     default is \code{report.html}.
#' @param interactive whether to generate interactive plots in the
#'     sample QC report if \code{QCreport = TRUE}.
#' @param results whether to write out the results for each QC steps
#'     in .txt files. The default is TRUE.
#' @param plotting whether to output the plots for each QC steps in
#'     .pdf files. the default is TRUE.
#' @param ... Arguments to be passed to other methods.
#' @export
#' @return a SeqSQC object with the filepath to the gds file which
#'     stores the genotype, the summary of samples and variants, and
#'     the QCresults including the sample annotation information and
#'     all QC results.

#' @details For \code{vfile} with more than one file names,
#'     \code{sampleQC} will merge all dataset together if they all
#'     contain the same samples. It is useful to combine
#'     genetic/genomic data together if VCF data is divided by
#'     chromosomes. \cr There are 3 columns in \code{sample.annot}
#'     file. col 1 is \code{sample} with sample ids, col 2 is
#'     \code{population} with values of "AFR/EUR/ASN/EAS/SAS", col 3
#'     is \code{gender} with values of "male/female".
#' @importFrom utils write.table
#' @examples
#' \dontrun{
#' infile <- system.file("extdata", "example_sub.vcf", package="SeqSQC")
#' sample.annot <- system.file("extdata", "sampleAnnotation.txt", package="SeqSQC")
#' cr <- system.file("extdata", "CCDS.Hs37.3.reduced_chr1.bed", package="SeqSQC")
#' outfile <- file.path(tempdir(), "testWrapUp")
#' seqfile <- sampleQC(vfile = infile, output = outfile, capture.region = cr,
#' sample.annot = sample.annot, format.data = "NGS", format.file = "vcf",
#' QCreport = TRUE, out.report="report.html", interactive = TRUE)
#' ## save(seqfile, file="seqfile.RData")
#'
#' load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
#' gfile <- system.file("extdata", "example.gds", package="SeqSQC")
#' seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
#' seqfile <- sampleQC(sfile = seqfile, output = outfile, QCreport = FALSE,
#' out.report="report.html", interactive = TRUE)
#' }
#' @author Qian Liu \email{qliu7@buffalo.edu}

sampleQC <- function(vfile = NULL, output="sampleqc",
                     capture.region = NULL, sample.annot = NULL,
                     LDprune = TRUE, vfile.restrict = FALSE,
                     slide.max.bp = 5e+05, ld.threshold = 0.3,
                     format.data = "NGS", format.file = "vcf",
                     QCreport = TRUE, out.report="report.html",
                     interactive = TRUE, results=TRUE, plotting=TRUE,
                     ...){

    ## check input
    if(inherits(vfile, "SeqSQC")){
        seqfile <- vfile
        format.data <- NULL
        format.file <- NULL
        vfile.restrict <- NULL
    }else{
        seqfile <- LoadVfile(vfile = vfile, output = output,
                             capture.region = capture.region,
                             sample.annot = sample.annot, ...)
    }
    fn <- gdsfile(seqfile)
    print(paste("gds file generated:", fn))
  
    sampleanno <- QCresult(seqfile)$sample.annot
    samples <- sampleanno$sample
    studyid <- sampleanno[sampleanno[,5] == "study", 1]
    
    ##############
    ## sample missing rate
    
    seqfile <- MissingRate(seqfile)
    res.mr <- QCresult(seqfile)$MissingRate
    res.study <- res.mr[res.mr$sample %in% studyid, ]
    prob.mr <- res.study[res.study$outlier == "Yes", ]
    remove.mr <- prob.mr[,1]
    remove.samples <- unique(remove.mr)
    if(results){
        write.table(res.mr,
                    file=paste0(dirname(output), "/result.missingrate.txt"),
                    quote=FALSE, sep="\t", row.names=FALSE)
    }
    if(plotting){
        p <- plotQC(seqfile, "MissingRate")
        ggsave(filename = paste0(dirname(output), "/plot.missingrate.pdf"), p)
    }
    
    ##############
    ## Sex Check

    ## check chrX variants
    gfile <- SeqOpen(seqfile)
    snp.chr <- read.gdsn(index.gdsn(gfile, "snp.chromosome"))
    closefn.gds(gfile)
    rm(gfile)
    if(!"X" %in% snp.chr){
        message("\nNOTE: Do not have chrX variants, skip sex check.\n")
        remove.samples <- remove.samples
    } else{
        seqfile <- SexCheck(seqfile)
        res.sexcheck <- QCresult(seqfile)$SexCheck
        table(res.sexcheck$sex, res.sexcheck$pred.sex)
        prob.sex <- res.sexcheck[res.sexcheck$sex != res.sexcheck$pred.sex &
                                 res.sexcheck$pred.sex != "0", ]
        remove.sex <- prob.sex[,1]
        remove.samples <- unique(c(remove.samples, remove.sex))
        if(results){
            write.table(res.sexcheck,
                        file=paste0(dirname(output), "/result.sexcheck.txt"),
                        quote=FALSE, sep="\t", row.names=FALSE)
        }
        if(plotting){
            p <- plotQC(seqfile, "SexCheck")
            ggsave(filename = paste0(dirname(output), "/plot.sexcheck.pdf"), p)
        }
    }
    
    ####################
    ## Inbreeding check
    
    seqfile <- Inbreeding(seqfile, remove.samples=remove.samples)
    res.inb <- QCresult(seqfile)$Inbreeding
    prob.inb <- res.inb[res.inb$outlier.5sd == "Yes" & !is.na(res.inb$outlier.5sd), ]
    remove.inb <- prob.inb[,1]
    remove.samples <- unique(c(remove.samples, remove.inb))
    if(results){
        write.table(res.inb,
                    file=paste0(dirname(output), "/result.inbreeding.txt"),
                    quote=FALSE, sep="\t", row.names=FALSE)
    }   
    ## plot Inbreeding.
    if(plotting){
        p <- plotQC(seqfile, "Inbreeding")
        ggsave(filename = paste0(dirname(output), "/plot.inbreeding.pdf"), p)
    }
    
    ########
    ## IBD
    
    seqfile <- IBDCheck(seqfile, remove.samples=remove.samples)
    res.ibd <- QCresult(seqfile)$IBD
    prob.ibd <- IBDRemove(seqfile)
    ibd.pairs <- prob.ibd$ibd.pairs
    if(nrow(ibd.pairs) > 0){
        ibd.pairs <- paste(ibd.pairs$id1, ibd.pairs$id2, sep=":")
    }else{
        ibd.pairs = NULL
    }
    remove.ibd <- prob.ibd$ibd.remove
    remove.samples <- unique(c(remove.samples, remove.ibd))
    if(results){
        write.table(res.ibd,
                    file=paste0(dirname(output), "/result.ibd.txt"),
                    quote=FALSE, sep="\t", row.names=FALSE)
    }
    ## plot IBD.
    if(plotting){
        p <- plotQC(seqfile, "IBD")
        ggsave(filename = paste0(dirname(output), "/plot.ibd.pdf"), p)
    }
    
    ########
    ## PCA

    seqfile <- PCACheck(seqfile, remove.samples=remove.samples)
    res.pca <- QCresult(seqfile)$PCA
    res.study <- res.pca[res.pca$sample %in% studyid, ]
    remove.pca <- res.study[res.study$pop != res.study$pred.pop, 1]

    if(results){
        write.table(res.pca,
                    file=paste0(dirname(output), "/result.pca.txt"),
                    quote=FALSE, sep="\t", row.names=FALSE)   
    }
    ## plot PCA.
    if(plotting){
        p <- plotQC(seqfile, "PCA")
        ggsave(filename = paste0(dirname(output), "/plot.pca.pdf"), p)
    }
    
    ################################
    ## summary of removed samples.
    problem.list <- problemList(seqfile)
    prob.list <- problem.list$prob.list
    rm.list <- problem.list$remove.list

    if(nrow(prob.list) != 0){
        a <- QCresult(seqfile)
        a$problem.list <- prob.list
        a$remove.list <- rm.list
        write.table(prob.list,
                    file=paste0(dirname(output), "/result.problemSamples.txt"),
                    quote=FALSE, sep="\t", row.names=FALSE)
        write.table(rm.list,
                    file=paste0(dirname(output), "/result.removeSamples.txt"),
                    quote=FALSE, sep="\t", row.names=FALSE,
                    col.names=FALSE)
    }

    if(QCreport){
        RenderReport(seqfile, output=out.report, interactive=interactive)
    }
    
    ###########################
    ## save the seqfile in SeqSQC
    ###########################
    return(seqfile)
}

#' SeqSQC
#'
#' Sample Quality Check for NGS Data.
#'
#' @name SeqSQC-package
#' @title Sample Quality Check for NGS Data using SeqSQC package
#' @import gdsfmt
#' @import SNPRelate
#' @import ExperimentHub
#' @author Qian Liu
#' @seealso
#' \code{\link{LoadVfile}} \cr
#' for data preparation; \cr
#' \code{\link{MissingRate}} \cr
#' \code{\link{PCACheck}} \cr
#' \code{\link{Inbreeding}} \cr
#' \code{\link{IBDCheck}} \cr
#' \code{\link{PCACheck}} \cr
#' for individual sample QC checks; \cr
#' \code{\link{problemList}} \cr
#' for the summary of problematic samples with reason and sample list to be removed; \cr
#' \code{\link{IBDRemove}} \cr
#' for the problematic sample pairs detected with cryptic relationship; \cr
#' \code{\link{RenderReport}} \cr
#' to generate the sample QC report; \cr
#' \code{\link{plotQC}} \cr
#' to generate the ggplot or interactive plots in html format for each individual QC check; \cr
#' \code{\link{sampleQC}} \cr
#' for wrapper of data preparation, all sample QC checks, QC result summary, and sample QC report.  

NULL

#' A data format to store genotype phenotype and sample QC results from SeqSQC.
#'
#' A SeqSQC object is a list of two objects. The first object
#' \code{gdsfile} is the filepath of the GDS (discussed in section
#' below) file which stores the genotype information from the original
#' VCF file. The second object \code{QCresult} is a list of sample
#' information and QC results, which include the dimension (# of
#' samples and variants), sample annotation, and QC results for sample
#' missing rate, sex check, inbreeding outlier check, IBD check, and
#' population outlier check.
#'
#' @slot gdsfile A character string for the filepath of the GDS file. 
#' @slot QCresult A list with sample information and sample QC results. 
#' @name SeqSQC-class
#' @rdname SeqSQC-class
#' @aliases SeqSQC-class
#' @exportClass SeqSQC

## create class definitions
setClass("SeqSQC",
         ## contains = "gds.class",
         slots = c(
             gdsfile = "character",
             QCresult = "SimpleList")
         )

#' SeqSQC object Constructor
#' @name SeqSQC-class
#' @rdname SeqSQC-class
#' @param gdsfile A character string for the filepath of the GDS file.
#' @param QCresult A list with sample information and sample QC results.
#' @export SeqSQC
SeqSQC <- function(gdsfile, QCresult=List()){
    new("SeqSQC", gdsfile = gdsfile, QCresult = QCresult)
}

#' gdsfile getter and setter.
#' @rdname SeqSQC-class
#' @exportMethod gdsfile
setGeneric("gdsfile", function(x) standardGeneric("gdsfile"))

#' @rdname SeqSQC-class
#' @exportMethod "gdsfile<-"
setGeneric("gdsfile<-", function(x, value) standardGeneric("gdsfile<-"))

#' QCresult getter and setter.
#' @rdname SeqSQC-class
#' @exportMethod QCresult
setGeneric("QCresult", function(x) standardGeneric("QCresult"))

#' @rdname SeqSQC-class
#' @exportMethod "QCresult<-"
setGeneric("QCresult<-", function(x, value) standardGeneric("QCresult<-"))

#' @rdname SeqSQC-class
#' @aliases gdsfile,SeqSQC-method
#' @param x an SeqSQCClass object.
#' @param value the new value for the SeqSQC object slots.
#' @examples
#' load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
#' gdsfile(seqfile)
#' @return The filepath to the gds file. 
setMethod("gdsfile", "SeqSQC",function(x) x@gdsfile)

#' @rdname SeqSQC-class
#' @aliases "gdsfile<-",SeqSQC-method
setReplaceMethod("gdsfile", "SeqSQC", function(x, value) {
    x <- initialize(x, gdsfile = value)
    x
})  

#' @rdname SeqSQC-class
#' @name QCresult
#' @aliases QCresult,SeqSQC-method
#' @examples
#' QCresult(seqfile)
setMethod("QCresult", "SeqSQC", function(x) x@QCresult)

#' @rdname SeqSQC-class
#' @aliases "QCresult<-",SeqSQC-method
setReplaceMethod("QCresult", "SeqSQC", function(x, value) {
    x <- initialize(x, QCresult = value)
    x
})

## set show methods
setMethod("show", "SeqSQC",
          function(object){
              res <- object@QCresult
              cat("SeqSQC\n")
              cat("gds file:", object@gdsfile, "\n")
              cat("summary: 87 benchmark samples,",
                  res$dimension[1]-87, "study samples,",
                  res$dimension[2], "variants\n")
              cat("QC result:", paste(names(res), collapse=", "), "\n")
          }
          )

## test the validity of objects
setValidity("SeqSQC",
            function(object)
            {
                dat <- openfn.gds(object@gdsfile)
                
                if (!inherits(dat, "gds.class")){
                    return("object should inherit from 'gds.class'.")
                }
                var.names <- ls.gdsn(dat)
                sampleanno <- read.gdsn(index.gdsn(dat, "sample.annot"))
                pops <- unique(sampleanno$population)
                closefn.gds(dat)
                
                if (!all(c("sample.id", "sample.annot",
                           "snp.id", "snp.chromosome", "snp.position", "snp.allele", 
                           "genotype") %in% var.names)){
                    return("sample.id, sample.annot, snp.id, snp.chromosome, snp.position, snp.allele, and genotype are required variables.")
                }
                
                if(!all(c("sample", "population", "gender", "relation", "group") %in% names(sampleanno))){
                    return("sample annotation with sample name, population, gender, relation and group info are required")
                }
                
                if (!all(pops %in% c("AFR", "EUR", "EAS", "SAS", "ASN"))){
                    return("the population group must be AFR, EUR, EAS, SAS and ASN.")
                }
                TRUE
            }
            )

#' Open the gds file in SeqSQC objects. 
#'
#' Function to open the gds file inside the SeqSQC object.
#' @param seqfile SeqSQC object, which has been merged with benchmark
#'     data.
#' @param readonly whether to open the gds file in read-only mode. If
#'     "FALSE", it is allowed to write data to the file. The default
#'     is TRUE.
#' @param allow.duplicate whether to allow to open a GDS file with
#'     read-only mode when it has been opened in the same R
#'     session. The default is FALSE.
#' @export
#' @return a gds file with the filepath in the input SeqSQC object.
#' @examples
#' library(gdsfmt)
#' load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
#' gfile <- system.file("extdata", "example.gds", package="SeqSQC")
#' seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
#' dat <- SeqOpen(seqfile)
#' dat
#' closefn.gds(dat)
#' @author Qian Liu \email{qliu7@@buffalo.edu}

SeqOpen <- function(seqfile, readonly=TRUE, allow.duplicate=FALSE)
{
    ## check
    fn <- gdsfile(seqfile)
    stopifnot(is.character(fn), length(seqfile)==1L)
    stopifnot(is.logical(readonly), length(readonly)==1L)
    stopifnot(is.logical(allow.duplicate), length(allow.duplicate)==1L)

    ## open the file
    dat <- openfn.gds(fn, readonly=readonly, allow.fork=TRUE,
                      allow.duplicate=allow.duplicate)
    return(dat)
}

#' Obtain the problematic sample list from IBD relatedness.
#'
#' Function to extract the related sample pairs from IBD results, and
#' to generate the sample list for removal from the related pairs
#' based on sample missing rate.
#' @param seqfile SeqSQC object, with IBD results.
#' @param all whether to check the IBD for all sample pairs (including
#'     the benchmark samples). The default is FALSE.
#' @return a list of 2 elements: \code{$ibd.pairs} is a data frame
#'     with 5 columns including sample names(id1, id2), IBD
#'     coefficients of k0 and k1, and kinship for samples with cryptic
#'     relatedness. \code{$ibd.remove} is a vector of samples to be
#'     removed, which are generated by extracting the sample with
#'     higher missing rate in each problematic sample pair.
#' @export
#' @examples
#' load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
#' gfile <- system.file("extdata", "example.gds", package="SeqSQC")
#' seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
#' seqfile <- IBDCheck(seqfile, remove.samples=NULL, LDprune=TRUE, missing.rate=0.1)
#' IBDRemove(seqfile)
#' @author Qian Liu \email{qliu7@@buffalo.edu}

IBDRemove <- function(seqfile, all = FALSE){

    ## check
    if (!inherits(seqfile, "SeqSQC")){
        return("object should inherit from 'SeqSQC'.")
    }
    if(!"IBD" %in% names(QCresult(seqfile))) stop("no IBD result.")

    sampleanno <- QCresult(seqfile)$sample.annot
    samples <- sampleanno$sample
    studyid <- sampleanno[sampleanno[,5] == "study", 1]
    res.ibd <- QCresult(seqfile)$IBD
    
    
    ## IBD results for only samples in study cohorts
    res.study <- res.ibd[res.ibd$id1 %in% studyid & res.ibd$id2 %in% studyid,]

    ## IBD results for all samples (including benchmark samples)
    ## res.study <- res.ibd

    ## ## IBD results for samples that are related with study samples
    ## res.study <- res.ibd[res.ibd$id1 %in% studyid | res.ibd$id2 %in% studyid,]

    ## check if any study sample is related with multiple samples
    ibd.related <- res.study[as.character(res.study$label) != as.character(res.study$pred.label) &
                             res.study$kin > 0.08, ]
    ibd.pairs <- ibd.related[, 1:5]
    
    if(nrow(ibd.related) > 0){
        ibd.related.tmp <- ibd.related
        remove.multi <- c()
        repeat{
            ibd.multi <- table(c(as.character(ibd.related.tmp$id1),
                                 as.character(ibd.related.tmp$id2)))
            ibd.multi <- sort(ibd.multi, decreasing=TRUE)[1]
            if(ibd.multi==1){break}
            remove.multi <- rbind(remove.multi, c(names(ibd.multi), unname(ibd.multi)))
            ibd.related.tmp <- ibd.related.tmp[!ibd.related.tmp$id1 %in% remove.multi &
                                               !ibd.related.tmp$id2 %in% remove.multi, ]
            if(nrow(ibd.related.tmp) == 0) {break}
        }
        ## remove the sample in the related pairs with higher missing rate. 
        message("remove samples in problematic pairs with higher missing rate...")
        if(! "MissingRate" %in% names(QCresult(seqfile))){
            seqfile <- MissingRate(seqfile)
        }
        mr <- QCresult(seqfile)$MissingRate
        mr <- mr[match(c(ibd.related$id1, ibd.related$id2), mr$sample), 1:2]
        ## }else{
        ##     gfile <- SeqOpen(seqfile, readonly=TRUE)
        ##     on.exit(closefn.gds(gfile))
        ##     gt <- readex.gdsn(index.gdsn(gfile, "genotype"),
        ##                       list(NULL, samples %in% c(ibd.related$id1, ibd.related$id2)))
        ##     mr <- apply(gt, 2, function(x) sum(! x %in% c(0, 1, 2))/length(x))
        ##     mr <- data.frame(sample = c(ibd.related$id1, ibd.related$id2), missingRate = mr, stringsAsFactors=FALSE)
        ##     ## closefn.gds(gfile)
        ## }    

        mr.rm <- function(x){
            mr.pair <- mr[match(x, mr$sample), 2]
            remove <- x[order(mr.pair, decreasing=TRUE)][1]
            return(remove)
        }
        remove.single <- unname(apply(ibd.related[, 1:2], 1, mr.rm))
        remove.ibd <- c(remove.multi[,1], remove.single)
    }else{
        remove.ibd=NULL
    }
    return(list(ibd.pairs=ibd.pairs, ibd.remove=remove.ibd))
}

        

#' Sample missing rate check with SeqSQC object input file.
#'
#' Function to calculate sample missing rate and to identify sample
#' outlier with high missing rate (> 0.1).
#' @param seqfile SeqSQC object, which includes the merged gds file
#'     for study cohort and benchmark.
#' @param remove.samples a vector of sample names for removal from
#'     missing rate check. Could be problematic samples identified
#'     from other QC steps, or user-defined samples.
#' @keywords MissingRate
#' @return a data frame with sample name, sample missing rate, and an
#'     indicator of whether the sample has a missing rate greater than
#'     0.1.
#' @details The value of the outlier column is set to NA for benchmark
#'     samples.
#' @export
#' @examples
#' load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
#' gfile <- system.file("extdata", "example.gds", package="SeqSQC")
#' seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
#' seqfile <- MissingRate(seqfile, remove.samples=NULL)
#' res.mr <- QCresult(seqfile)$MissingRate
#' tail(res.mr)
#' @author Qian Liu \email{qliu7@@buffalo.edu}

MissingRate <- function(seqfile, remove.samples=NULL){

    ## check
    if (!inherits(seqfile, "SeqSQC")){
        return("object should inherit from 'SeqSQC'.")
    }

    message("calculating sample missing rates...")
    
    gfile <- SeqOpen(seqfile, readonly=TRUE)
    samples <- read.gdsn(index.gdsn(gfile, "sample.id"))
    sampleanno <- read.gdsn(index.gdsn(gfile, "sample.annot"))

    ## sample filters. (remove prespecified "remove.samples", and only
    ## keep samples within the study population)
    study.pop <- unique(sampleanno[sampleanno$group == "study", "population"])
    if (length(study.pop) > 1)
        stop("Study samples should be single population, please prepare input file accordingly.")
    if (study.pop == "ASN") study.pop <- c("EAS", "SAS", "ASN")
    
    if (!is.null(remove.samples)){
        flag <- sampleanno$population %in% study.pop & !samples %in% remove.samples
    } else {
        flag <- sampleanno$population %in% study.pop
    }
    sample.mr <- samples[flag]
    
    gt <- index.gdsn(gfile, "genotype")
    mr <- apply.gdsn(gt, 2, function(x) sum(! x %in% c(0, 1, 2))/length(x))
    mr <- unlist(mr)
    mr <- mr[samples %in% sample.mr]
    outlier <- ifelse(mr > 0.1, "Yes", "No")
    res.mr <- data.frame(sample = sample.mr, missingRate = mr,
                         outlier = outlier, stringsAsFactors = FALSE)

    skip.idx <- sampleanno[match(sample.mr, sampleanno$sample), 5] != "study"
    res.mr[skip.idx, "outlier"] <- NA

    closefn.gds(gfile)

    ## return the SeqSQC file with updated QC results.
    a <- QCresult(seqfile)
    a$MissingRate <- res.mr
    ## QCresult(seqfile)$MissingRate <-  res.mr
    outfile <- SeqSQC(gdsfile = gdsfile(seqfile), QCresult = a)
    return(outfile)
}

#' Sample gender check with SeqSQC object input file.
#'
#' Function to calculate the X chromosome inbreeding coefficient and
#' to predict sample gender.
#' @param seqfile SeqSQC object, which includes the merged gds file
#'     for study cohort and benchmark.
#' @param remove.samples a vector of sample names for removal from sex
#'     check. Could be problematic samples identified from previous QC
#'     steps, or user-defined samples.
#' @param missing.rate to use the SNPs with "<= \code{missing.rate}"
#'     only; if NaN, no threshold. By default, we use
#'     \code{missing.rate = 0.1} to filter out variants with missing
#'     rate greater than 10\%.
#' @param ss.cutoff the minimum sample size (300 by default) to apply
#'     the MAF filter. This sample size is the sum of study samples
#'     and the benchmark samples of the same population as the study
#'     cohort.
#' @param maf to use the SNPs with ">= \code{maf}" if sample size
#'     defined in above argument is greater than \code{ss.cutoff};
#'     otherwise NaN is used by default for no MAF threshold.
#' @param ... Arguments to be passed to other methods.
#' @keywords SexCheck
#' @return a data frame with sample name, reported gender, x
#'     chromosome inbreeding coefficient, and predicted gender.
#' @details Samples are predicted to be female or male if the
#'     inbreeding coefficient is below 0.2, or greater than 0.8,
#'     respectively. The samples with discordant reported gender and
#'     predicted gender are considered as problematic. When the
#'     inbreeding coefficient is within the range of [0.2, 0.8], “0”
#'     is shown in the column of \code{pred.sex} to indicate ambiguous
#'     gender, which is not considered as problematic.
#' @export
#' @examples
#' load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
#' gfile <- system.file("extdata", "example.gds", package="SeqSQC")
#' seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
#' seqfile <- SexCheck(seqfile, remove.samples=NULL, missing.rate=0.1)
#' res.sexc <- QCresult(seqfile)$SexCheck
#' tail(res.sexc)
#' @author Qian Liu \email{qliu7@@buffalo.edu}

SexCheck <- function(seqfile, remove.samples=NULL, missing.rate = 0.1,
                     ss.cutoff = 300, maf = 0.01, ...){

    ## check
    if (!inherits(seqfile, "SeqSQC")){
        return("object should inherit from 'SeqSQC'.")
    }

    message("calculating sex inbreeding ...")
    
    gfile <- SeqOpen(seqfile, readonly=TRUE)
    
    nds <- c("sample.id", "sample.annot", "snp.chromosome", "snp.position", "snp.id") 
    allnds <- lapply(nds, function(x) read.gdsn(index.gdsn(gfile, x)))
    names(allnds) <- c("samples", "sampleanno", "snp.chr", "snp.pos", "snps")

     ## sample filters. (remove prespecified "remove.samples", and only keep samples within the study population)
    study.pop <- unique(allnds$sampleanno[allnds$sampleanno$group == "study", "population"])
    if (length(study.pop) > 1)
        stop("Study samples should be single population, please prepare input file accordingly.")

    if(study.pop == "ASN") study.pop <- c("EAS", "SAS", "ASN")
    
    if(!is.null(remove.samples)){
        flag <- allnds$sampleanno$population %in% study.pop & !allnds$samples %in% remove.samples
    }else{
        flag <- allnds$sampleanno$population %in% study.pop
    }
    sample.sex <- allnds$samples[flag]
        
    ## SNP filters. (only keep X variants, and remove X variants in pseudo-autosomal region.)
    flag <- allnds$snp.chr == "X"
    snp.x <- allnds$snps[flag]
    snp.chr.x <- allnds$snp.chr[flag]
    snp.pos.x <- allnds$snp.pos[flag]

    ## remove pseudo-autosomal region in X chromosome.
    grx <- GRanges(snp.chr.x, IRanges(snp.pos.x, snp.pos.x))
    par <- GRanges(c("X", "X"), IRanges(start=c(60001, 154931044), end=c(2699520, 155260560)))
    ov <- findOverlaps(grx, par)
    par.rm <- unique(queryHits(ov))
    snp.x.final <- snp.x[-par.rm]

    ## calculate population-specific sex inbreeding coefficient.

    ## use maf filter for variants when sample size >= 300.
    if (length(sample.sex) >= ss.cutoff){
        sexinb <- snpgdsIndInb(gfile, autosome.only=FALSE,
                               sample.id=sample.sex,
                               snp.id=snp.x.final, maf=maf,
                               missing.rate=missing.rate, ...)
    }else{
        sexinb <- snpgdsIndInb(gfile, autosome.only=FALSE,
                               sample.id=sample.sex,
                               snp.id=snp.x.final, maf=NaN,
                               missing.rate=missing.rate, ...)
    }
    closefn.gds(gfile)
    
    ## extract gender info
    sex <- allnds$sampleanno[match(sample.sex, allnds$sampleanno[,1]), 3]
    res.sexcheck <- data.frame(sample=sample.sex, sex=sex,
                               sexinb=sexinb$inbreeding,
                               stringsAsFactors=FALSE)
    
    ## gender prediction.
    pred <- ifelse(res.sexcheck$sexinb > 0.8, "male",
            ifelse(res.sexcheck$sexinb < 0.2, "female", 0))
    res.sexcheck$pred.sex <- pred
    
    ## return the SeqSQC file with updated QC results.
    a <- QCresult(seqfile)
    a$SexCheck <- res.sexcheck
    outfile <- SeqSQC(gdsfile = gdsfile(seqfile), QCresult = a)
    return(outfile)
}

#' Population outlier check with SeqSQC object input file.
#'
#' Function to perform principle component analysis for all samples
#' and to infer sample ancestry.
#' @param seqfile SeqSQC object, which includes the merged gds file
#'     for study cohort and benchmark.
#' @param remove.samples a vector of sample names for removal from PCA
#'     calculation. Could be problematic samples identified from
#'     previous QC steps, or user-defined samples.
#' @param npcs the number principle components to use for the
#'     population prediction in SVM model. The default value is 4, and
#'     it is required to be <= 10.
#' @param LDprune whether to use LD-pruned snp set, the default is
#'     TRUE.
#' @param missing.rate to use the SNPs with "<= \code{missing.rate}"
#'     only; if NaN, no threshold. By default, we use
#'     \code{missing.rate = 0.1} to filter out variants with missing
#'     rate greater than 10\%.
#' @param ss.cutoff the minimum sample size (300 by default) to apply
#'     the MAF filter. This sample size is the sum of study samples
#'     and the benchmark samples of the same population as the study
#'     cohort.
#' @param maf to use the SNPs with ">= \code{maf}" if sample size
#'     defined in above argument is greater than \code{ss.cutoff};
#'     otherwise NaN is used by default for no MAF threshold.
#' @param hwe to use the SNPs with Hardy-Weinberg equilibrium p >=
#'     \code{hwe} if sample size defined in above argument is greater
#'     than \code{ss.cutoff}; otherwise no hwe threshold. The default
#'     is 1e-6.
#' @param ... Arguments to be passed to other methods.
#' @keywords PCA
#' @return a data frame with sample name, reported population, data
#'     resource (benchmark vs study cohort), the first four
#'     eigenvectors and the predicted population.
#' @details Using LD-pruned autosomal variants (by default), we
#'     calculate the eigenvectors and eigenvalues for principle
#'     component analysis (PCA). We use the benchmark samples as
#'     training dataset, and predict the population group for each
#'     sample in the study cohort based on the top four
#'     eigenvectors. Samples with discordant predicted and
#'     self-reported population groups are considered problematic. The
#'     function \code{PCACheck} performs the PCA analysis and
#'     identifies population outliers in study cohort.
#' @importFrom stats predict as.formula
#' @export
#' @examples
#' load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
#' gfile <- system.file("extdata", "example.gds", package="SeqSQC")
#' seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
#' seqfile <- PCACheck(seqfile, remove.samples=NULL, LDprune=TRUE, missing.rate=0.1)
#' res.pca <- QCresult(seqfile)$PCA
#' tail(res.pca)
#' @author Qian Liu \email{qliu7@@buffalo.edu}

PCACheck <- function(seqfile, remove.samples = NULL, npcs = 4,
                     LDprune = TRUE, missing.rate = 0.1,
                     ss.cutoff = 300, maf = 0.01, hwe = 1e-6, ...){
    
    ## check
    if (!inherits(seqfile, "SeqSQC")){
        return("object should inherit from 'SeqSQC'.")
    }
    stopifnot(npcs <= 10)
    
    message("calculating sample principle components ...")
    
    gfile <- SeqOpen(seqfile, readonly=TRUE)
    
    nds <- c("sample.id", "sample.annot", "snp.id") 
    allnds <- lapply(nds, function(x) read.gdsn(index.gdsn(gfile, x)))
    names(allnds) <- c("samples", "sampleanno", "snp.id")
    
    ## samples <- read.gdsn(index.gdsn(gfile, "sample.id"))
    ## snp.id <- read.gdsn(index.gdsn(gfile, "snp.id"))
    ## sampleanno <- read.gdsn(index.gdsn(gfile, "sample.annot"))

    ## remove samples for family related samples from 1kg, and other
    ## problematic samples from previous QC steps.
    study.pop <- unique(allnds$sampleanno[allnds$sampleanno$group == "study", "population"])
    if(length(study.pop) > 1)
        stop("Study samples should be single population, please prepare input file accordingly.")
    if(study.pop == "ASN") study.pop <- c("EAS", "SAS", "ASN")
    
    sample.relate <- allnds$sampleanno[allnds$sampleanno[,5]=="fam", 1]
    
    if(!is.null(remove.samples)){
        flag <- (allnds$sampleanno$group != "study" | allnds$sampleanno$population %in% study.pop) &
            ! allnds$samples %in% c(sample.relate, remove.samples)
    }else{
        flag <- (allnds$sampleanno$group != "study" | allnds$sampleanno$population %in% study.pop) &
            ! allnds$samples %in% sample.relate
    }
    sample.pca <- allnds$samples[flag]
    
    ## add hwe filter for variants when sample size >= 300.
    if (length(sample.pca) >= ss.cutoff){
        snp.hwe <- snpgdsHWE(gfile, sample.id=sample.pca)
        hwe.idx <- snp.hwe > hwe & !is.na(snp.hwe)
    }else{
        hwe.idx <- rep(TRUE, length(allnds$snp.id))
    }
    
    ## use LDpruned SNPs if LDprune == TRUE.
    if(LDprune){
        ld <- read.gdsn(index.gdsn(gfile, "snp.annot/LDprune"))
    }else{
        ld <- rep(TRUE, length(allnds$snp.id))
    }
    
    ## SNP filters in together. (HWE+LDprune)
    snp.idx <- ld & hwe.idx
    
    ## use maf filter if sample size >= 300. 
    if (length(sample.pca) >= ss.cutoff){
        pca <- snpgdsPCA(gfile, sample.id=sample.pca,
                         snp.id=allnds$snp.id[snp.idx],
                         missing.rate=missing.rate, maf=maf, ...)
    }else{
        pca <- snpgdsPCA(gfile, sample.id=sample.pca,
                         snp.id=allnds$snp.id[snp.idx],
                         missing.rate=missing.rate, maf=NaN, ...)
    }   
    closefn.gds(gfile)

    ## Make a data.frame and save result.
    res.pca <- data.frame(
        sample = pca$sample.id,
        pop = factor(allnds$sampleanno[,2])[match(pca$sample.id, allnds$sampleanno[,1])],
        type = factor(allnds$sampleanno[,5])[match(pca$sample.id, allnds$sampleanno[,1])],
        eval = pca$eigenval,
        ## pca$eigenvect,
        EV1 = pca$eigenvect[,1],    # the first eigenvector
        EV2 = pca$eigenvect[,2],    # the second eigenvector
        EV3 = pca$eigenvect[,3],
        EV4 = pca$eigenvect[,4],
        EV5 = pca$eigenvect[,5],
        EV6 = pca$eigenvect[,6],
        EV7 = pca$eigenvect[,7],
        EV8 = pca$eigenvect[,8],
        EV9 = pca$eigenvect[,9],
        EV10 = pca$eigenvect[,10], 
        stringsAsFactors = FALSE
    )
    ## Fst estimation
    ## pop.code <- factor(allnds$sampleanno[,2][match(pca$sample.id, allnds$sampleanno[,1])])
    ## a <- snpgdsFst(gfile, sample.id=sample.pca, population=pop.code, method="W&C84")
    
    ## ++++++++++++
    ## prediction 
    ## ++++++++++++
    
    ## use "svm" for classification, use the first 4 EigenVectors for prediction.
    ind.pop <- res.pca$type=="pop"
    
    ## 1. doc. 2. npcs<=10
    xs <- paste0("EV", seq_len(npcs))
    fmla <- as.formula(paste("pop ~ ", paste(xs, collapse=" + ")))
    model <- svm(fmla, data=res.pca[ind.pop, ], probability=FALSE, kernel="linear")
    pred.pop <- predict(model, res.pca[, 4+seq_len(npcs)], probability=FALSE)
    ## }  ?? 
    res.pca <- cbind(res.pca[, c(1:3, 4+seq_len(npcs))], pred.pop)

    ## model <- svm(pop ~ EV1 + EV2 + EV3 + EV4, data=res.pca[ind.pop,
    ## ], probability=FALSE, kernel="linear") pred.pop <-
    ## predict(model, res.pca[, 5:8], probability=FALSE) } res.pca <-
    ## cbind(res.pca[, c(1:3, 5:8)], pred.pop)

    ## for "ASN" samples, prediction of either "EAS" or "SAS" would be
    ## correct and summarized as "ASN".
    if("ASN" %in% levels(res.pca$pop)){
        res.pca[res.pca$pop == "ASN" & res.pca$pred.pop %in% c("EAS", "SAS"), "pred.pop"] <- "ASN"
    }
    ##res.pca <- cbind(res.pca, pred.pop, attributes(pred.pop)$probabilities)

    ## return the SeqSQC file with updated gds file and QC result.
    a <- QCresult(seqfile)
    a$PCA <- res.pca
    outfile <- SeqSQC(gdsfile = gdsfile(seqfile), QCresult = a)
    return(outfile)
}

#' Sample inbreeding check with SeqSQC object input file.
#'
#' Function to calculate population-specific inbreeding coefficients,
#' and to predict inbreeding outliers that are five standard deviation
#' beyond the mean.
#' @param seqfile SeqSQC object, which includes the merged gds file
#'     for study cohort and benchmark.
#' @param remove.samples a vector of sample names for removal from
#'     inbreeding coefficient calculation. Could be problematic
#'     samples identified from previous QC steps, or user-defined
#'     samples.
#' @param LDprune whether to use LD-pruned snp set. The default is
#'     TRUE.
#' 
#' @param missing.rate to use the SNPs with "<= \code{missing.rate}"
#'     only; if NaN, no threshold. By default, we use
#'     \code{missing.rate = 0.1} to filter out variants with missing
#'     rate greater than 10\%.
#' @param ss.cutoff the minimum sample size (300 by default) to apply
#'     the MAF filter. This sample size is the sum of study samples
#'     and the benchmark samples of the same population as the study
#'     cohort.
#' @param maf to use the SNPs with ">= \code{maf}" if sample size
#'     defined in above argument is greater than \code{ss.cutoff};
#'     otherwise NaN is used by default for no MAF threshold.
#' @param hwe to use the SNPs with Hardy-Weinberg equilibrium p >=
#'     \code{hwe} if sample size defined in above argument is greater
#'     than \code{ss.cutoff}; otherwise no hwe threshold. The default
#'     is 1e-6.
#' @param ... Arguments to be passed to other methods.
#' @keywords inbreeding
#' @return a data frame with sample name, inbreeding coefficient, and
#'     an indicator of whether the inbreeding coefficient is five
#'     standard deviation beyond the mean.
#' @details Using LD-pruned variants (by default), we calculate the
#'     inbreeding coefficients for each sample in the study cohort and
#'     for benchmark samples of the same population as the study
#'     cohort. Samples with inbreeding coefficients that are five
#'     standard deviations beyond the mean are considered problematic
#'     and are shown as "Yes" in the column of
#'     \code{outlier.5sd}. Benchmark samples in this column are set to
#'     be “NA”.
#' @export
#' @examples
#' load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
#' gfile <- system.file("extdata", "example.gds", package="SeqSQC")
#' seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
#' seqfile <- Inbreeding(seqfile, remove.samples=NULL, LDprune=TRUE, missing.rate=0.1)
#' res.inb <- QCresult(seqfile)$Inbreeding
#' tail(res.inb)
#' @author Qian Liu \email{qliu7@@buffalo.edu}

Inbreeding <- function(seqfile, remove.samples=NULL, LDprune=TRUE,
                       missing.rate=0.1, ss.cutoff = 300, maf = 0.01,
                       hwe = 1e-6, ...){

    ## check
    if (!inherits(seqfile, "SeqSQC")){
        return("object should inherit from 'SeqSQC'.")
    }
    message("calculating inbreeding coefficients ...")
    
    gfile <- SeqOpen(seqfile, readonly=TRUE)
    
    nds <- c("sample.id", "sample.annot", "snp.id") 
    allnds <- lapply(nds, function(x) read.gdsn(index.gdsn(gfile, x)))
    names(allnds) <- c("samples", "sampleanno", "snp.id")
    
    ## sample filters. (remove prespecified "remove.samples", and only
    ## keep samples within the study population)
    study.pop <- unique(allnds$sampleanno[allnds$sampleanno$group == "study", "population"])
    if (length(study.pop) > 1)
        stop("Study samples should be single population, please prepare input file accordingly.")
    
    if(study.pop == "ASN") study.pop <- c("EAS", "SAS", "ASN")
    
    if(!is.null(remove.samples)){
        flag <- allnds$sampleanno$population %in% study.pop & !allnds$samples %in% remove.samples
    }else{
        flag <- allnds$sampleanno$population %in% study.pop
    }
    sample.inb <- allnds$samples[flag]
    
    ## calculate population-specific inbreeding coefficient.

    ## add hwe filter for variants when sample size >= 300.
    if (length(sample.inb) >= ss.cutoff){
        snp.hwe <- snpgdsHWE(gfile, sample.id=sample.inb)
        hwe.idx <- snp.hwe > hwe & !is.na(snp.hwe)
    }else{
        hwe.idx <- rep(TRUE, length(allnds$snp.id))
    }
    
    ## use LDpruned SNPs if LDprune == TRUE.
    if(LDprune){
        ld <- read.gdsn(index.gdsn(gfile, "snp.annot/LDprune"))
    }else{
        ld <- rep(TRUE, length(allnds$snp.id))
    }
    
    ## SNP filters in together. (HWE+LDprune)
    snp.idx <- ld & hwe.idx
    
    ## use maf filter if sample size >= 300.
    if (length(sample.inb) >= ss.cutoff){ 
        rv <- snpgdsIndInb(gfile, snp.id=allnds$snp.id[snp.idx],
                           sample.id=sample.inb, maf=maf,
                           missing.rate=missing.rate, ...)
    }else{
        rv <- snpgdsIndInb(gfile, snp.id=allnds$snp.id[snp.idx],
                           sample.id=sample.inb, maf=NaN,
                           missing.rate=missing.rate, ...)
    }
    closefn.gds(gfile)
        
    ## outliers
    m <- mean(rv$inbreeding)
    s <- sd(rv$inbreeding)
    coef <- 5
    outlier.5sd <- ifelse(rv$inbreeding < m-coef*s | rv$inbreeding > m+coef*s,
                          "Yes", "No")
    
    res.inb <- data.frame(sample=sample.inb,
                          inbreeding=rv$inbreeding, outlier.5sd,
                          stringsAsFactors=FALSE)

    ## set all outlier.* to be "NA" for 1000g benchmark samples. (We
    ## only use the 1000g samples to help identify inbreeding outliers
    ## from study cohort, and calculate inbreeding coef together with
    ## study samples within the sample population)
    skip.idx <- allnds$sampleanno[match(sample.inb, allnds$sampleanno$sample), 5] != "study"
    res.inb[skip.idx, "outlier.5sd"] <- NA
    
    ## return the SeqSQC file with updated QC results.
    a <- QCresult(seqfile)
    a$Inbreeding <- res.inb
    outfile <- SeqSQC(gdsfile = gdsfile(seqfile), QCresult = a)
    return(outfile)
}

library(AnnotationHubData)

meta <- data.frame(
    Title = "DNA-Sequencing dataset from the 1000 Genomes Project",
    Description = paste0("DNA-seq data from the 1000 Genomes Project ",
                         "containing 22 AFR, 22 EAS, 21 EUR and 22 SAS samples. ",
                         "there are eight known related pairs including ",
                         "four parent-offspring pairs, two full-sibling pairs ",
                         "and two half-sibling or avuncular pairs in this dataset, ",
                         "which is saved as a Genomic Data Structure (GDS) file."),
    BiocVersion = "3.6",
    Genome = "GRCh37",
    SourceType = "VCF",
    SourceUrl = paste0("ftp://ftp-trace.ncbi.nih.gov/1000genomes/ftp/release/20130502",
                       "ftp://ftp-trace.ncbi.nih.gov/1000genomes/ftp/release/20130502/supporting/related_samples_vcf/", collapse=","),
    SourceVersion = "May 02 2013",
    Species = "Homo sapiens",
    TaxonomyId = 9606,
    Coordinate_1_based = TRUE,
    DataProvider = "1000 Genomes Project",
    Maintainer = "Qian Liu <qliu7@buffalo.edu>",
    RDataPath = "SeqSQC/benchmark_1000genomes.gds",
    Location_Prefix = "http://s3.amazonaws.com/experimenthub/",
    RDataClass = "gds.class",
    DispatchClass = "GDS",
    ResourceName = "benchmark_1000genomes.gds"
)

## Not run:

## Write the data out and put in the inst/extdata directory.
write.csv(meta, file="SeqSQC/inst/extdata/metadata.csv", row.names=FALSE)

## Test the validity of metadata.csv with readMetadataCsv():
readMetadataFromCsv("SeqSQC")
## End(Not run)

plotSexCheck <- function(res.qc, interactive=FALSE){
    if (any(res.qc$sex == "")) {
        warning("Please provide sex info for all study samples!")
        res.qc <- res.qc[res.qc$sex != "", ]
    }
    res.pop.ord <- res.qc
    prob.ind <- res.pop.ord$sex == "female" & res.pop.ord$pred.sex == "male" |
        res.pop.ord$sex == "male" & res.pop.ord$pred.sex == "female"
    prob.sex <- res.pop.ord[prob.ind, ]
    
    p <- ggplot(data=res.pop.ord, aes(x=sample, y=sexinb))
    p <- p + geom_point(aes(colour=sex, shape=type))
    p <- p + scale_colour_discrete(drop = FALSE)
    p <- p + labs(col="Self-reported gender",
                  shape="Type",
                  x="Sample",
                  y="Sex inbreeding coefficient")
    p <- p + geom_hline(yintercept=c(0.2, 0.8), colour="grey")
    if(nrow(prob.sex) > 0){
        p <- p + geom_point(data=prob.sex,
                            aes(x=sample, y=sexinb), shape=1,
                            colour="red", size=4)
    }
    p <- p + theme_classic()
    p <- p + theme(axis.text.x=element_blank(),
                   axis.ticks.x=element_blank())  
    ## p <- p + theme(legend.position = c(0.2, 0.5))
    if(interactive){
        return(ggplotly(p))
    }else{
        return(p)
    }
}

#' Generate the problematic sample list.
#'
#' generate the problematic sample list from QC steps that have been
#' done, and provide each problematic sample with a reason for removal
#' (high missing rate, gender mismatch, inbreeding outlier, cryptic
#' relationship or population outlier).
#' @param seqfile SeqSQC object with sample QC results.
#' @export
#' @return a list of 2 datasets: 1) a data frame with 2 columns:
#'     \code{sample} for problematic sample name, and
#'     \code{remove.reason} for the reason of removing the sample. 2)
#'     a data frame with 1 column \code{sample} for problematic
#'     samples to be removed.
#' @examples
#' load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
#' problemList(seqfile)
#' @author Qian Liu \email{qliu7@@buffalo.edu}

problemList <- function(seqfile){

    sampleanno <- QCresult(seqfile)$sample.annot
    samples <- sampleanno$sample
    studyid <- sampleanno[sampleanno[,5] == "study", 1]

    if("MissingRate" %in% names(QCresult(seqfile))){
        res.mr <- QCresult(seqfile)$MissingRate
        res.study <- res.mr[res.mr$sample %in% studyid, ]
        prob.mr <- res.study[res.study$outlier == "Yes", ]
        remove.mr <- prob.mr[,1]
    }else{
        remove.mr <- NULL
    }
        
    if("SexCheck" %in% names(QCresult(seqfile))){
        res.sexcheck <- QCresult(seqfile)$SexCheck
        prob.sex <- res.sexcheck[res.sexcheck$sex == "female" & res.sexcheck$pred.sex == "male" |
                                 res.sexcheck$sex == "male" & res.sexcheck$pred.sex == "female", ]
        remove.sex <- prob.sex[,1]
    }else{
        remove.sex <- NULL
    }

    if("Inbreeding" %in% names(QCresult(seqfile))){
    res.inb <- QCresult(seqfile)$Inbreeding
    prob.inb <- res.inb[res.inb$outlier.5sd == "Yes" & !is.na(res.inb$outlier.5sd), ]
    remove.inb <- prob.inb[,1]
    }else{
        remove.inb <- NULL
    }

    if("IBD" %in% names(QCresult(seqfile))){
        res.ibd <- QCresult(seqfile)$IBD
        prob.ibd <- IBDRemove(seqfile)
        ibd.pairs <- prob.ibd$ibd.pairs
        if(nrow(ibd.pairs) > 0){
            ibd.pairs <- paste(ibd.pairs$id1, ibd.pairs$id2, sep=":")
        }else{
            ibd.pairs = NULL
        }
        remove.ibd <- prob.ibd$ibd.remove
    }else{
        ibd.pairs <- NULL
        remove.ibd <- NULL
    }
    
    if("PCA" %in% names(QCresult(seqfile))){
        res.pca <- QCresult(seqfile)$PCA
        res.study <- res.pca[res.pca$sample %in% studyid, ]
        remove.pca <- res.study[res.study$pop != res.study$pred.pop, 1]
    }else{
        remove.pca <- NULL
    }
    
    prob.list <- data.frame(
        sample=c(remove.mr, remove.sex, remove.inb, ibd.pairs, remove.pca),
        ## remove=c(remove.mr, remove.sex, remove.inb, remove.ibd, remove.pca),
        remove.reason=c(
            rep("high missing rate", length(remove.mr)),
            rep("gender mismatch", length(remove.sex)),
            rep("inbreeding outlier", length(remove.inb)),
            rep("cryptic relationship", length(ibd.pairs)),
            rep("population outlier", length(remove.pca))),
        stringsAsFactors=FALSE
    )
    remove.list <- data.frame(sample = c(remove.mr, remove.sex, remove.inb, remove.ibd, remove.pca),
                              stringsAsFactors=FALSE)
    if(nrow(prob.list) != 0){
        a <- QCresult(seqfile)
        a$problem.list <- list(prob.list = prob.list, remove.list = remove.list)
        return(a$problem.list)
    } else {
        NULL
    }
}

readgdsn <- function(gfile, node){
    if (!inherits(gfile, "gds.class")){
        return("object should inherit from 'gds.class'.")
    }
    stopifnot(class(node) == "character")
    stopifnot(node %in% ls.gdsn(gfile))

    gdsn <- read.gdsn(index.gdsn(gfile, node))
    gdsn
}

plotIBD <- function(res.qc, interactive = FALSE){

    centers <- data.frame(k0=c(0, 0, 0.25, 0.5, 1),
                          k1=c(0, 1, 0.5, 0.5, 0),
                          label=factor(c("DU", "PO", "FS", "HF", "UN"))
                          )
    prob.ind <- as.character(res.qc$relation) == "UN" &
        as.character(res.qc$pred.label) == "Related"
    prob.ibd <- res.qc[prob.ind, ]

    p <- ggplot(data=res.qc, aes(x=k0, y=k1))
    p <- p + geom_point(aes(shape=type, colour=relation))
    p <- p + scale_colour_discrete(drop = FALSE)
    p <- p + geom_point(aes(colour=label), shape=3, data=centers)
    cols <- c(brewer.pal(n=9, name="Set1")[c(1,3,5,8)], "black")
    p <- p + scale_color_manual(values=cols)
    if(nrow(prob.ibd) > 0){
        p <- p + geom_point(data=prob.ibd, aes(x=k0, y=k1),
                            shape=1, colour="red", size=4)
    }
    p <- p + theme_classic()
    p <- p + theme(legend.position = c(0.9, 0.8))
    p <- p + labs(x="K0 (probability that two individuals share 0 alleles)",
                  y="K1 (probability that two individuals share 1 alleles)",
                  col="Relation",
                  shape="Type")

    if(interactive){
        return(ggplotly(p))
    }else{
        return(p)
    }
}

plotInbreeding <- function(res.qc, interactive = FALSE, sdcoef = 5){
    res.pop.ord <- res.qc
    inb <- res.pop.ord$inbreeding
    m <- mean(inb)
    sd <- sd(inb)
    sd1 <- m + sdcoef*sd
    sd2 <- m - sdcoef*sd
    res.pop.ord$outlier <- ifelse(res.pop.ord$inbreeding > sd1 | res.pop.ord$inbreeding < sd2,
                                  "Yes", "No")
    res.pop.ord$problematic <- factor(res.pop.ord$outlier, levels=c("Yes", "No"))
    
    p <- ggplot(data=res.pop.ord, aes(x=sample, y=inbreeding))
    p <- p + geom_point(aes(shape=type, colour=problematic))
    p <- p + scale_colour_discrete(drop = FALSE)
    p <- p + geom_hline(yintercept=c(sd1, sd2), colour="grey")
    p <- p + theme_classic()
    p <- p + labs(col="Problematic",
                  shape="Type",
                  x="Sample",
                  y="Autosomal inbreeding coefficient")
    ## p <- p + theme(legend.position = c(0.1, 0.2))
    p <- p + theme(axis.text.x=element_blank(),
                   axis.ticks.x=element_blank())  

    if(interactive){
        return(ggplotly(p))
    }else{
        return(p)
    }
}

#' example gds file used in vignette.
#'
#' This gds file contains genotype and phenotype for 92 whole-genome
#' sequenced samples captured by CCDS region.  This is a merged
#' dataset of the 87 benchmark samples and the 5 study samples (all
#' are assembled from the 1000 Genomes Project).  The meta info for
#' these 92 samples includes sample name, pupulation, age, relation
#' note and group info (benchmark or study).
#'
#' @name example.gds
#' @docType data
#' @author Qian Liu \email{qliu7@@buffalo.edu}
#' @keywords datasets
NULL

#' Example vcf file used in vignette.
#'
#' This vcf file contains only a subset (1000 lines of variants) of
#' the original vcf file for the 5 study samples (examples assembled
#' from the 1000 Genomes Project).  This is to be used as a runnable
#' example in the function of \code{LoadVfile} and \code{sampleQC} in
#' the vignette.
#'
#' @name example_sub.vcf
#' @docType data
#' @author Qian Liu \email{qliu7@@buffalo.edu}
#' @keywords datasets
NULL

#' Example capture region file used in vignette.
#'
#' This \code{.bed} file contains only CCDS capture region only in
#' chromosome 1, which is meant to be used together with the
#' \code{example_sub.vcf} as a runnable example in the function of
#' \code{LoadVfile} and \code{sampleQC} in the vignette.
#'
#' @name CCDS.Hs37.3.reduced_chr1.bed
#' @docType data
#' @author Qian Liu \email{qliu7@@buffalo.edu}
#' @keywords datasets
NULL

#' Example SeqSQC file used in vignette.
#'
#' The SeqSQC object is a list of two objects. The first object
#' \code{gdsfile} is the filepath of the "example.gds" file which
#' stores the genotype and meta info of the example data merged with
#' the benchmark data.  The second object \code{QCresult} contains the
#' data dimensions (# of samples and variants), sample annotation, and
#' QC results for sample missing rate, sex check, inbreeding outlier
#' check, IBD check, and population outlier check.
#' @name example.seqfile.Rdata
#' @docType data
#' @author Qian Liu \email{qliu7@@buffalo.edu}
#' @keywords datasets
NULL

#' Sample annotation file for the example data used in vignette.
#'
#' This sample annotation file is a required input from the user when
#' using SeqSQC.  It includes the sample info with sample name stored
#' in the column of \code{sample}, the population info stored in the
#' column of \code{population}, and the gender info stored in the
#' column of \code{gender}.  The \code{population} column must be a in
#' the format of "AFR/EUR/ASN/EAS/SAS". The \code{gender} column must
#' be in the format of "female/male". This file is meant to be used
#' together with the \code{example_sub.vcf} as a runnable example in
#' the function of \code{LoadVfile} and \code{sampleQC} in the
#' vignette.
#' @name sampleAnnotation.txt
#' @docType data
#' @author Qian Liu \email{qliu7@@buffalo.edu}
#' @keywords datasets
NULL

plotQCData <- function(seqfile, QCstep=NULL){

    ## input check
    if (!inherits(seqfile, "SeqSQC")){
        return("object should inherit from 'SeqSQC'.")
    }
    if(!QCstep %in% names(QCresult(seqfile))) stop(paste("no QC result for", QCstep))

    if(QCstep == "MissingRate"){
        res.qc <- QCresult(seqfile)$MissingRate
    }else if(QCstep == "SexCheck"){
        res.qc <- QCresult(seqfile)$SexCheck
    }else if(QCstep == "Inbreeding"){
        res.qc <- QCresult(seqfile)$Inbreeding
    }else if(QCstep == "IBD"){
        res.qc <- QCresult(seqfile)$IBD
    }else if(QCstep == "PCA"){
        res.qc <- QCresult(seqfile)$PCA
    }
    sampleanno <- QCresult(seqfile)$sample.annot
    
    if(QCstep %in% c("MissingRate", "SexCheck", "Inbreeding")){
        type <- sampleanno[match(res.qc$sample, sampleanno[,1]), 5]
        type[type=="fam"] <- "pop"
        res.qc$type <- factor(type, levels=c("pop", "study"), labels=c("benchmark", "study"))
        pop <- sampleanno[match(res.qc$sample, sampleanno[,1]), 2]
        res.qc$population <- factor(pop)
        rm(type, pop)
        
        ## by default, "study.pop" is a single population for missing
        ## rate, sexcheck and inbreeding. If not single population,
        ## give error message.
        study.pop <- unique(sampleanno[sampleanno$group == "study", "population"])
        if(length(study.pop)>1) stop("Study samples should be single population.")
        
        ## population specific calculation for missingrate, sexcheck and inbreeding. 
        if(study.pop == "ASN"){
            res.pop <- res.qc[res.qc$population %in% c("EAS", "SAS", "ASN"), ]
        }else{
            res.pop <- res.qc[res.qc$population == study.pop, ]
        }

        b <- res.pop[res.pop$type == "benchmark", ]
        s <- res.pop[res.pop$type == "study" & res.pop$population == study.pop, ]
        res.pop.ord <- rbind(b[order(b$population), ], s)
        res.pop.ord$sample <- factor(res.pop.ord$sample, levels=res.pop.ord$sample)
        res.qc <- res.pop.ord
        
    }else if(QCstep == "IBD"){
        studyid <- sampleanno[sampleanno[,5] == "study", 1]
        
        res.qc$label <- as.character(res.qc$label)
        res.qc$relation <- factor(res.qc$label, levels=c("DU", "PO", "FS", "HF", "UN"))
        
        ind.study <- res.qc$id1 %in% studyid | res.qc$id2 %in% studyid
        names(res.qc)[6] <- "type"
        res.qc$type[ind.study] <- "study"
        res.qc$type[!ind.study] <- "benchmark"
        
    }else if(QCstep == "PCA"){
        res.qc$type <- as.character(res.qc$type)
        res.qc$type[res.qc$type == "pop"] <- "benchmark"
        res.qc$type <- factor(res.qc$type)
    }
    res.qc
}

context("sample meta info")

test_that("population input is correct", {
    sample.annot <- system.file("extdata", "sampleAnnotation.txt", package="SeqSQC")
    meta <- read.table(sample.annot, header=T)
    expect_true(all(meta[,2] %in% c("AFR", "EUR", "EAS", "SAS", "ASN")))
})

## to do: 
- "IBDRemove.R". if QCresult(seqfile)$MissingRate is missing, recalculate the missing rate by opening the gfile connection. 
todo: if missing, then run `seqfile <- MissingRate(seqfile)` and extract the missing rate info. -- done!
- "plotInbreeding", interactive, xlab, ylab, refer to plotSexCheck.  -- done.
- "dataset_docum.R". to document all files that are under inst/extdata/...  -- done for all 5 files. (except "sampleQCreport.Rmd" and "metadata.csv")
- "PCACheck.R", add an argumnent of "npcs=4" for the # of pc axis to use for pop prediction in SVM. -- done. 

---
title: sample QC report
subtitle:
date: "`r Sys.Date()`"
params: 
  seqfile: NA
  interactive: TRUE
---

# 1. summary of samples

```{r, echo=FALSE}
seqfile <- params$seqfile
interactive <- params$interactive
```

```{r, echo=FALSE}
sampleanno <- seqfile@QCresult$sample.annot
ind.study <- sampleanno[,5] == "study"
anno.study <- sampleanno[ind.study, ]
## table(anno.study$population)
## table(anno.study$gender)
```
We have a total of `r nrow(anno.study)` study samples from the population of `r unique(anno.study$population)`, with `r unname(table(anno.study$gender)[names(table(anno.study$gender)) %in% "female"])` female samples and `r unname(table(anno.study$gender)[names(table(anno.study$gender)) %in% "male"])` male samples. 

# 2. Sample quality check

Here is the list of all QC steps we have done. 

```{r, echo=FALSE}
(qcresults <- names(QCresult(seqfile))[names(QCresult(seqfile)) %in% c("MissingRate", "SexCheck", "Inbreeding", "IBD", "PCA")])
```

## 2.1 sample missing rate check

Samples with high missing rate (>0.1) are marked as "Yes" in the column of `outlier` and should be removed. The value of the `outlier` column is set to "NA" for benchmark samples.

```{r, echo=FALSE, warning = FALSE}
if("MissingRate" %in% qcresults){
    res.mr <- QCresult(seqfile)$MissingRate
    plotQC(seqfile, QCstep = "MissingRate", interactive = interactive)    
}else{
    print("MissingRate check is not done yet!")
    res.mr <- NULL
}
```
There are `r sum(res.mr$outlier %in% "Yes")` problematic samples due to high missing rate.

## 2.2 Sex check

Samples are predicted to be female or male if the inbreeding coefficient is below 0.2, or greater than 0.8. The samples with discordant reported gender and predicted gender are considered as problematic. When the inbreeding coefficient is within the range of [0.2, 0.8], “0” will be shown in the column of `pred.sex` to indicate ambiguous gender, which would not be considered as problematic.  

```{r, echo=FALSE, warning = FALSE}
if("SexCheck" %in% qcresults){
    res.sex <- QCresult(seqfile)$SexCheck
    tail(res.sex)
    table(res.sex$sex, res.sex$pred.sex)
    ## sample with gender mismatch
    (prob.sex <- res.sex[res.sex$sex == "female" & res.sex$pred.sex == "male" | res.sex$sex == "male" & res.sex$pred.sex == "female", ])
    plotQC(seqfile, QCstep = "SexCheck", interactive = interactive)
}else{
    print("Sex check is not done yet!")
    prob.sex <- NULL
}
```
There are `r nrow(prob.sex)` problematic samples due to gender mismatch.   

## 2.3 Inbreeding check

By using LD-pruned variants (by default), we calculate the inbreeding coefficients for each sample in the study cohort and for benchmark samples of the same population as the study cohort. Samples with inbreeding coefficients that are five standard deviations beyond the mean are considered problematic and shows "Yes" in the column of `outlier.5sd`. Benchmark samples in this column are set to be “NA”.  

The estimate of inbreeding coefficient could be negative due to random sampling error, but a strongly negative value could reflect other factors, e.g. sample contamination events.

```{r, echo=FALSE, warning = FALSE}
if("Inbreeding" %in% qcresults){
    res.inb <- QCresult(seqfile)$Inbreeding
    tail(res.inb)
    table(res.inb$outlier.5sd)
    ## sample with inbreeding outlier
    (prob.inb <- res.inb[res.inb$outlier.5sd == "Yes" & !is.na(res.inb$outlier.5sd), ])
    plotQC(seqfile, QCstep = "Inbreeding", interactive = interactive)
}else{
    print("Inbreeding outlier check is not done yet!")
    prob.inb <- NULL
}
```
There are `r nrow(prob.inb)` problematic samples that are inbreeding outliers.   

## 2.4 IBD check

By using LD-pruned variants (by default), we calculate the IBD coefficients for all sample pairs, and then predict related sample pairs in study cohort using the support vector machine (SVM) method with linear kernel and the known relatedness embedded in benchmark data as training set.   

Sample pairs with discordant self-reported and predicted relationship are considered as problematic. All predicted related pairs are also required to have coefficient of kinship >= 0.08. The sample with higher missing rate in each related pair is selected for removal from further analysis by function of `IBDRemove`.

```{r, echo=FALSE, warning = FALSE}
if("IBD" %in% qcresults){
    res.ibd <- QCresult(seqfile)$IBD
    head(res.ibd)
    table(res.ibd$label, res.ibd$pred.label)
    ## sample with cryptic relatedness
    (prob.ibd <- IBDRemove(seqfile))
    plotQC(seqfile, QCstep = "IBD", interactive = interactive)
}else{
    print("IBD check is not done yet!")
    prob.ibd <- NULL
}
```
There are `r nrow(prob.ibd$ibd.pairs)` problematic sample pairs with cryptic relationship.

## 2.5 Population outlier check

By using LD-pruned variants (by default), and benchmark samples as training dataset, we predict the ancestry for each sample in the study cohort using the top four principle components by support machine (SVM) with linear kernel.  

Samples with discordant predicted and self-reported population groups are considered problematic. The function `PCA` performs the PCA analysis and identifies population outliers in study cohort.

```{r, echo=FALSE, warning = FALSE}
if("PCA" %in% qcresults){
    res.pca <- QCresult(seqfile)$PCA
    tail(res.pca)
    studyid <- sampleanno[ind.study, 1]
    res.study <- res.pca[res.pca$sample %in% studyid, ]
    (prob.pca <- res.study[res.study$pop != res.study$pred.pop, ])
    plotQC(seqfile, QCstep = "PCA", interactive = interactive)
}else{
    print("Population outlier check is not done yet!")
    prob.pca <- NULL
}
```
There are `r nrow(prob.pca)` problematic samples that are detected to be population outliers.   

# 3. Summary of problematic samples.

Here is the summary list of problematic samples. Problematic samples are identified by SeqSQC due to different reasons of high missing rate, gender mismatch, inbreeding outlier, cryptic relationship or population outlier.

```{r, echo=FALSE}
if ("problem.list" %in% names(QCresult(seqfile))){
    (probs <- QCresult(seqfile)$problem.list)
    (prob.remove <- QCresult(seqfile)$remove.list)
}else{
    (probs <- problemList(seqfile))
}
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SexCheck.R
\name{SexCheck}
\alias{SexCheck}
\title{Sample gender check with SeqSQC object input file.}
\usage{
SexCheck(
  seqfile,
  remove.samples = NULL,
  missing.rate = 0.1,
  ss.cutoff = 300,
  maf = 0.01,
  ...
)
}
\arguments{
\item{seqfile}{SeqSQC object, which includes the merged gds file
for study cohort and benchmark.}

\item{remove.samples}{a vector of sample names for removal from sex
check. Could be problematic samples identified from previous QC
steps, or user-defined samples.}

\item{missing.rate}{to use the SNPs with "<= \code{missing.rate}"
only; if NaN, no threshold. By default, we use
\code{missing.rate = 0.1} to filter out variants with missing
rate greater than 10\%.}

\item{ss.cutoff}{the minimum sample size (300 by default) to apply
the MAF filter. This sample size is the sum of study samples
and the benchmark samples of the same population as the study
cohort.}

\item{maf}{to use the SNPs with ">= \code{maf}" if sample size
defined in above argument is greater than \code{ss.cutoff};
otherwise NaN is used by default for no MAF threshold.}

\item{...}{Arguments to be passed to other methods.}
}
\value{
a data frame with sample name, reported gender, x
    chromosome inbreeding coefficient, and predicted gender.
}
\description{
Function to calculate the X chromosome inbreeding coefficient and
to predict sample gender.
}
\details{
Samples are predicted to be female or male if the
    inbreeding coefficient is below 0.2, or greater than 0.8,
    respectively. The samples with discordant reported gender and
    predicted gender are considered as problematic. When the
    inbreeding coefficient is within the range of [0.2, 0.8], “0”
    is shown in the column of \code{pred.sex} to indicate ambiguous
    gender, which is not considered as problematic.
}
\examples{
load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
gfile <- system.file("extdata", "example.gds", package="SeqSQC")
seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
seqfile <- SexCheck(seqfile, remove.samples=NULL, missing.rate=0.1)
res.sexc <- QCresult(seqfile)$SexCheck
tail(res.sexc)
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}
\keyword{SexCheck}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IBDRemove.R
\name{IBDRemove}
\alias{IBDRemove}
\title{Obtain the problematic sample list from IBD relatedness.}
\usage{
IBDRemove(seqfile, all = FALSE)
}
\arguments{
\item{seqfile}{SeqSQC object, with IBD results.}

\item{all}{whether to check the IBD for all sample pairs (including
the benchmark samples). The default is FALSE.}
}
\value{
a list of 2 elements: \code{$ibd.pairs} is a data frame
    with 5 columns including sample names(id1, id2), IBD
    coefficients of k0 and k1, and kinship for samples with cryptic
    relatedness. \code{$ibd.remove} is a vector of samples to be
    removed, which are generated by extracting the sample with
    higher missing rate in each problematic sample pair.
}
\description{
Function to extract the related sample pairs from IBD results, and
to generate the sample list for removal from the related pairs
based on sample missing rate.
}
\examples{
load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
gfile <- system.file("extdata", "example.gds", package="SeqSQC")
seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
seqfile <- IBDCheck(seqfile, remove.samples=NULL, LDprune=TRUE, missing.rate=0.1)
IBDRemove(seqfile)
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SeqOpen.R
\name{SeqOpen}
\alias{SeqOpen}
\title{Open the gds file in SeqSQC objects.}
\usage{
SeqOpen(seqfile, readonly = TRUE, allow.duplicate = FALSE)
}
\arguments{
\item{seqfile}{SeqSQC object, which has been merged with benchmark
data.}

\item{readonly}{whether to open the gds file in read-only mode. If
"FALSE", it is allowed to write data to the file. The default
is TRUE.}

\item{allow.duplicate}{whether to allow to open a GDS file with
read-only mode when it has been opened in the same R
session. The default is FALSE.}
}
\value{
a gds file with the filepath in the input SeqSQC object.
}
\description{
Function to open the gds file inside the SeqSQC object.
}
\examples{
library(gdsfmt)
load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
gfile <- system.file("extdata", "example.gds", package="SeqSQC")
seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
dat <- SeqOpen(seqfile)
dat
closefn.gds(dat)
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PCACheck.R
\name{PCACheck}
\alias{PCACheck}
\title{Population outlier check with SeqSQC object input file.}
\usage{
PCACheck(
  seqfile,
  remove.samples = NULL,
  npcs = 4,
  LDprune = TRUE,
  missing.rate = 0.1,
  ss.cutoff = 300,
  maf = 0.01,
  hwe = 1e-06,
  ...
)
}
\arguments{
\item{seqfile}{SeqSQC object, which includes the merged gds file
for study cohort and benchmark.}

\item{remove.samples}{a vector of sample names for removal from PCA
calculation. Could be problematic samples identified from
previous QC steps, or user-defined samples.}

\item{npcs}{the number principle components to use for the
population prediction in SVM model. The default value is 4, and
it is required to be <= 10.}

\item{LDprune}{whether to use LD-pruned snp set, the default is
TRUE.}

\item{missing.rate}{to use the SNPs with "<= \code{missing.rate}"
only; if NaN, no threshold. By default, we use
\code{missing.rate = 0.1} to filter out variants with missing
rate greater than 10\%.}

\item{ss.cutoff}{the minimum sample size (300 by default) to apply
the MAF filter. This sample size is the sum of study samples
and the benchmark samples of the same population as the study
cohort.}

\item{maf}{to use the SNPs with ">= \code{maf}" if sample size
defined in above argument is greater than \code{ss.cutoff};
otherwise NaN is used by default for no MAF threshold.}

\item{hwe}{to use the SNPs with Hardy-Weinberg equilibrium p >=
\code{hwe} if sample size defined in above argument is greater
than \code{ss.cutoff}; otherwise no hwe threshold. The default
is 1e-6.}

\item{...}{Arguments to be passed to other methods.}
}
\value{
a data frame with sample name, reported population, data
    resource (benchmark vs study cohort), the first four
    eigenvectors and the predicted population.
}
\description{
Function to perform principle component analysis for all samples
and to infer sample ancestry.
}
\details{
Using LD-pruned autosomal variants (by default), we
    calculate the eigenvectors and eigenvalues for principle
    component analysis (PCA). We use the benchmark samples as
    training dataset, and predict the population group for each
    sample in the study cohort based on the top four
    eigenvectors. Samples with discordant predicted and
    self-reported population groups are considered problematic. The
    function \code{PCACheck} performs the PCA analysis and
    identifies population outliers in study cohort.
}
\examples{
load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
gfile <- system.file("extdata", "example.gds", package="SeqSQC")
seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
seqfile <- PCACheck(seqfile, remove.samples=NULL, LDprune=TRUE, missing.rate=0.1)
res.pca <- QCresult(seqfile)$PCA
tail(res.pca)
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}
\keyword{PCA}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4.R
\docType{class}
\name{SeqSQC-class}
\alias{SeqSQC-class}
\alias{SeqSQC}
\alias{gdsfile}
\alias{gdsfile<-}
\alias{QCresult}
\alias{QCresult<-}
\alias{gdsfile,SeqSQC-method}
\alias{gdsfile<-,SeqSQC-method}
\alias{"gdsfile<-",SeqSQC-method}
\alias{QCresult,SeqSQC-method}
\alias{QCresult<-,SeqSQC-method}
\alias{"QCresult<-",SeqSQC-method}
\title{A data format to store genotype phenotype and sample QC results from SeqSQC.}
\usage{
SeqSQC(gdsfile, QCresult = List())

gdsfile(x)

gdsfile(x) <- value

QCresult(x)

QCresult(x) <- value

\S4method{gdsfile}{SeqSQC}(x)

\S4method{gdsfile}{SeqSQC}(x) <- value

\S4method{QCresult}{SeqSQC}(x)

\S4method{QCresult}{SeqSQC}(x) <- value
}
\arguments{
\item{gdsfile}{A character string for the filepath of the GDS file.}

\item{QCresult}{A list with sample information and sample QC results.}

\item{x}{an SeqSQCClass object.}

\item{value}{the new value for the SeqSQC object slots.}
}
\value{
The filepath to the gds file.
}
\description{
A SeqSQC object is a list of two objects. The first object
\code{gdsfile} is the filepath of the GDS (discussed in section
below) file which stores the genotype information from the original
VCF file. The second object \code{QCresult} is a list of sample
information and QC results, which include the dimension (# of
samples and variants), sample annotation, and QC results for sample
missing rate, sex check, inbreeding outlier check, IBD check, and
population outlier check.
}
\section{Slots}{

\describe{
\item{\code{gdsfile}}{A character string for the filepath of the GDS file.}

\item{\code{QCresult}}{A list with sample information and sample QC results.}
}}

\examples{
load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
gdsfile(seqfile)
QCresult(seqfile)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SeqSQC-package.R
\name{SeqSQC-package}
\alias{SeqSQC-package}
\title{Sample Quality Check for NGS Data using SeqSQC package}
\description{
SeqSQC
}
\details{
Sample Quality Check for NGS Data.
}
\seealso{
\code{\link{LoadVfile}} \cr
for data preparation; \cr
\code{\link{MissingRate}} \cr
\code{\link{PCACheck}} \cr
\code{\link{Inbreeding}} \cr
\code{\link{IBDCheck}} \cr
\code{\link{PCACheck}} \cr
for individual sample QC checks; \cr
\code{\link{problemList}} \cr
for the summary of problematic samples with reason and sample list to be removed; \cr
\code{\link{IBDRemove}} \cr
for the problematic sample pairs detected with cryptic relationship; \cr
\code{\link{RenderReport}} \cr
to generate the sample QC report; \cr
\code{\link{plotQC}} \cr
to generate the ggplot or interactive plots in html format for each individual QC check; \cr
\code{\link{sampleQC}} \cr
for wrapper of data preparation, all sample QC checks, QC result summary, and sample QC report.
}
\author{
Qian Liu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RenderReport.R
\name{RenderReport}
\alias{RenderReport}
\title{Render the rmarkdown file for generating a sample QC report.}
\usage{
RenderReport(input, output, interactive = TRUE)
}
\arguments{
\item{input}{SeqSQC object with QC results.}

\item{output}{a character string to define the file name for the QC
report.}

\item{interactive}{whether to generate interative plots in the
report. The default is TRUE.}
}
\value{
Will incure the rendering of the rmarkdown file for
    generating the sample QC report. The report will return to the
    file denoted in \code{output} in the function.
}
\description{
Function to render the pre-compiled rmarkdown file to generate the
sample QC report.
}
\examples{
load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
RenderReport(seqfile, output="report.html", interactive=FALSE)
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MissingRate.R
\name{MissingRate}
\alias{MissingRate}
\title{Sample missing rate check with SeqSQC object input file.}
\usage{
MissingRate(seqfile, remove.samples = NULL)
}
\arguments{
\item{seqfile}{SeqSQC object, which includes the merged gds file
for study cohort and benchmark.}

\item{remove.samples}{a vector of sample names for removal from
missing rate check. Could be problematic samples identified
from other QC steps, or user-defined samples.}
}
\value{
a data frame with sample name, sample missing rate, and an
    indicator of whether the sample has a missing rate greater than
    0.1.
}
\description{
Function to calculate sample missing rate and to identify sample
outlier with high missing rate (> 0.1).
}
\details{
The value of the outlier column is set to NA for benchmark
    samples.
}
\examples{
load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
gfile <- system.file("extdata", "example.gds", package="SeqSQC")
seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
seqfile <- MissingRate(seqfile, remove.samples=NULL)
res.mr <- QCresult(seqfile)$MissingRate
tail(res.mr)
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}
\keyword{MissingRate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_docum.R
\docType{data}
\name{example_sub.vcf}
\alias{example_sub.vcf}
\title{Example vcf file used in vignette.}
\description{
This vcf file contains only a subset (1000 lines of variants) of
the original vcf file for the 5 study samples (examples assembled
from the 1000 Genomes Project).  This is to be used as a runnable
example in the function of \code{LoadVfile} and \code{sampleQC} in
the vignette.
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_docum.R
\docType{data}
\name{sampleAnnotation.txt}
\alias{sampleAnnotation.txt}
\title{Sample annotation file for the example data used in vignette.}
\description{
This sample annotation file is a required input from the user when
using SeqSQC.  It includes the sample info with sample name stored
in the column of \code{sample}, the population info stored in the
column of \code{population}, and the gender info stored in the
column of \code{gender}.  The \code{population} column must be a in
the format of "AFR/EUR/ASN/EAS/SAS". The \code{gender} column must
be in the format of "female/male". This file is meant to be used
together with the \code{example_sub.vcf} as a runnable example in
the function of \code{LoadVfile} and \code{sampleQC} in the
vignette.
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/problemList.R
\name{problemList}
\alias{problemList}
\title{Generate the problematic sample list.}
\usage{
problemList(seqfile)
}
\arguments{
\item{seqfile}{SeqSQC object with sample QC results.}
}
\value{
a list of 2 datasets: 1) a data frame with 2 columns:
    \code{sample} for problematic sample name, and
    \code{remove.reason} for the reason of removing the sample. 2)
    a data frame with 1 column \code{sample} for problematic
    samples to be removed.
}
\description{
generate the problematic sample list from QC steps that have been
done, and provide each problematic sample with a reason for removal
(high missing rate, gender mismatch, inbreeding outlier, cryptic
relationship or population outlier).
}
\examples{
load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
problemList(seqfile)
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Inbreeding.R
\name{Inbreeding}
\alias{Inbreeding}
\title{Sample inbreeding check with SeqSQC object input file.}
\usage{
Inbreeding(
  seqfile,
  remove.samples = NULL,
  LDprune = TRUE,
  missing.rate = 0.1,
  ss.cutoff = 300,
  maf = 0.01,
  hwe = 1e-06,
  ...
)
}
\arguments{
\item{seqfile}{SeqSQC object, which includes the merged gds file
for study cohort and benchmark.}

\item{remove.samples}{a vector of sample names for removal from
inbreeding coefficient calculation. Could be problematic
samples identified from previous QC steps, or user-defined
samples.}

\item{LDprune}{whether to use LD-pruned snp set. The default is
TRUE.}

\item{missing.rate}{to use the SNPs with "<= \code{missing.rate}"
only; if NaN, no threshold. By default, we use
\code{missing.rate = 0.1} to filter out variants with missing
rate greater than 10\%.}

\item{ss.cutoff}{the minimum sample size (300 by default) to apply
the MAF filter. This sample size is the sum of study samples
and the benchmark samples of the same population as the study
cohort.}

\item{maf}{to use the SNPs with ">= \code{maf}" if sample size
defined in above argument is greater than \code{ss.cutoff};
otherwise NaN is used by default for no MAF threshold.}

\item{hwe}{to use the SNPs with Hardy-Weinberg equilibrium p >=
\code{hwe} if sample size defined in above argument is greater
than \code{ss.cutoff}; otherwise no hwe threshold. The default
is 1e-6.}

\item{...}{Arguments to be passed to other methods.}
}
\value{
a data frame with sample name, inbreeding coefficient, and
    an indicator of whether the inbreeding coefficient is five
    standard deviation beyond the mean.
}
\description{
Function to calculate population-specific inbreeding coefficients,
and to predict inbreeding outliers that are five standard deviation
beyond the mean.
}
\details{
Using LD-pruned variants (by default), we calculate the
    inbreeding coefficients for each sample in the study cohort and
    for benchmark samples of the same population as the study
    cohort. Samples with inbreeding coefficients that are five
    standard deviations beyond the mean are considered problematic
    and are shown as "Yes" in the column of
    \code{outlier.5sd}. Benchmark samples in this column are set to
    be “NA”.
}
\examples{
load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
gfile <- system.file("extdata", "example.gds", package="SeqSQC")
seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
seqfile <- Inbreeding(seqfile, remove.samples=NULL, LDprune=TRUE, missing.rate=0.1)
res.inb <- QCresult(seqfile)$Inbreeding
tail(res.inb)
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}
\keyword{inbreeding}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampleQC.R
\name{sampleQC}
\alias{sampleQC}
\title{The wrap-up function for sample QC of sequencing/GWAS data.}
\usage{
sampleQC(
  vfile = NULL,
  output = "sampleqc",
  capture.region = NULL,
  sample.annot = NULL,
  LDprune = TRUE,
  vfile.restrict = FALSE,
  slide.max.bp = 5e+05,
  ld.threshold = 0.3,
  format.data = "NGS",
  format.file = "vcf",
  QCreport = TRUE,
  out.report = "report.html",
  interactive = TRUE,
  results = TRUE,
  plotting = TRUE,
  ...
)
}
\arguments{
\item{vfile}{vcf or PLINK input file (ped/map/bed/bim/fam with same
basename). The default is NULL. Vfile could be a vector of
character strings, see details. Could also take file in
\code{SeqSQC} object generated from \code{LoadVfile}.}

\item{output}{a character string for name of merged data of SeqSQC
object. The \code{dirname(output)} would be used as the
directory to save the QC result and plots. The default is
\code{sampleqc} in the working directory.}

\item{capture.region}{the BED file of sequencing capture
regions. The default is NULL. For exome-sequencing data, the
capture region file must be provided.}

\item{sample.annot}{sample annotation file with 3 columns (with
header) in the order of sample id, sample population and sex
info. The default is NULL.}

\item{LDprune}{whether to use LD-pruned snp set. The default is
TRUE.}

\item{vfile.restrict}{whether the input vcf or plink file has
already been restricted by capture region. The default is
FALSE.}

\item{slide.max.bp}{the window size of SNPs when calculating
linkage disequilibrium. The default is 5e+05.}

\item{ld.threshold}{the r^2 threshold for LD-based SNP pruning if
\code{LDprune = TRUE}. The default is 0.3.}

\item{format.data}{the data source. The default is \code{NGS} for
sequencing data.}

\item{format.file}{the data format. The default is \code{vcf}.}

\item{QCreport}{Whether to generate the sample QC report in html
format.}

\item{out.report}{the file name for the sample QC report. The
default is \code{report.html}.}

\item{interactive}{whether to generate interactive plots in the
sample QC report if \code{QCreport = TRUE}.}

\item{results}{whether to write out the results for each QC steps
in .txt files. The default is TRUE.}

\item{plotting}{whether to output the plots for each QC steps in
.pdf files. the default is TRUE.}

\item{...}{Arguments to be passed to other methods.}
}
\value{
a SeqSQC object with the filepath to the gds file which
    stores the genotype, the summary of samples and variants, and
    the QCresults including the sample annotation information and
    all QC results.
}
\description{
A wrap-up function for sample QC. It reads in the variant genotypes
in vcf/PLINK format, merges study cohort with benchmark data, and
performs sample QC for the merged dataset.
}
\details{
For \code{vfile} with more than one file names,
    \code{sampleQC} will merge all dataset together if they all
    contain the same samples. It is useful to combine
    genetic/genomic data together if VCF data is divided by
    chromosomes. \cr There are 3 columns in \code{sample.annot}
    file. col 1 is \code{sample} with sample ids, col 2 is
    \code{population} with values of "AFR/EUR/ASN/EAS/SAS", col 3
    is \code{gender} with values of "male/female".
}
\examples{
\dontrun{
infile <- system.file("extdata", "example_sub.vcf", package="SeqSQC")
sample.annot <- system.file("extdata", "sampleAnnotation.txt", package="SeqSQC")
cr <- system.file("extdata", "CCDS.Hs37.3.reduced_chr1.bed", package="SeqSQC")
outfile <- file.path(tempdir(), "testWrapUp")
seqfile <- sampleQC(vfile = infile, output = outfile, capture.region = cr,
sample.annot = sample.annot, format.data = "NGS", format.file = "vcf",
QCreport = TRUE, out.report="report.html", interactive = TRUE)
## save(seqfile, file="seqfile.RData")

load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
gfile <- system.file("extdata", "example.gds", package="SeqSQC")
seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
seqfile <- sampleQC(sfile = seqfile, output = outfile, QCreport = FALSE,
out.report="report.html", interactive = TRUE)
}
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_docum.R
\docType{data}
\name{CCDS.Hs37.3.reduced_chr1.bed}
\alias{CCDS.Hs37.3.reduced_chr1.bed}
\title{Example capture region file used in vignette.}
\description{
This \code{.bed} file contains only CCDS capture region only in
chromosome 1, which is meant to be used together with the
\code{example_sub.vcf} as a runnable example in the function of
\code{LoadVfile} and \code{sampleQC} in the vignette.
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_docum.R
\docType{data}
\name{example.seqfile.Rdata}
\alias{example.seqfile.Rdata}
\title{Example SeqSQC file used in vignette.}
\description{
The SeqSQC object is a list of two objects. The first object
\code{gdsfile} is the filepath of the "example.gds" file which
stores the genotype and meta info of the example data merged with
the benchmark data.  The second object \code{QCresult} contains the
data dimensions (# of samples and variants), sample annotation, and
QC results for sample missing rate, sex check, inbreeding outlier
check, IBD check, and population outlier check.
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IBDCheck.R
\name{IBDCheck}
\alias{IBDCheck}
\title{Sample relationship check with SeqSQC object input file.}
\usage{
IBDCheck(
  seqfile,
  remove.samples = NULL,
  LDprune = TRUE,
  kin.filter = TRUE,
  missing.rate = 0.1,
  ss.cutoff = 300,
  maf = 0.01,
  hwe = 1e-06,
  ...
)
}
\arguments{
\item{seqfile}{SeqSQC object, which includes the merged gds file
for study cohort and benchmark.}

\item{remove.samples}{a vector of sample names for removal from IBD
calculation. Could be problematic samples identified from
previous QC steps, or user-defined samples.}

\item{LDprune}{whether to use LD-pruned snp set. The default is
TRUE.}

\item{kin.filter}{whether to use "kinship coefficient >= 0.08" as
the additional criteria for related samples. The default is
TRUE.}

\item{missing.rate}{to use the SNPs with "<= \code{missing.rate}"
only; if NaN, no threshold. By default, we use
\code{missing.rate = 0.1} to filter out variants with missing
rate greater than 10\%.}

\item{ss.cutoff}{the minimum sample size (300 by default) to apply
the MAF filter. This sample size is the sum of study samples
and the benchmark samples of the same population as the study
cohort.}

\item{maf}{to use the SNPs with ">= \code{maf}" if sample size
defined in above argument is greater than \code{ss.cutoff};
otherwise NaN is used by default for no MAF threshold.}

\item{hwe}{to use the SNPs with Hardy-Weinberg equilibrium p >=
\code{hwe} if sample size defined in above argument is greater
than \code{ss.cutoff}; otherwise no hwe threshold. The default
is 1e-6.}

\item{...}{Arguments to be passed to other methods.}
}
\value{
a data frame with sample names, the descent coefficients of
    k0, k1 and kinship, self-reported relationship and predicted
    relationship for each pair of samples.
}
\description{
Function to calculate the IBD coefficients for all sample pairs and
to predict related sample pairs in study cohort.
}
\details{
Using LD-pruned variants (by default), we calculate the
    IBD coefficients for all sample pairs, and then predict related
    sample pairs in study cohort using the support vector machine
    (SVM) method with linear kernel and the known relatedness
    embedded in benchmark data as training set. \cr Sample pairs
    with discordant self-reported and predicted relationship are
    considered as problematic. All predicted related pairs are also
    required to have coefficient of kinship >= 0.08 by default. The
    sample with higher missing rate in each related pair is
    selected for removal from further analysis by function of
    \code{IBDRemove}.
}
\examples{
load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
gfile <- system.file("extdata", "example.gds", package="SeqSQC")
seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
seqfile <- IBDCheck(seqfile, remove.samples=NULL, LDprune=TRUE, missing.rate=0.1)
res.ibd <- QCresult(seqfile)$IBD
tail(res.ibd)
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}
\keyword{IBD}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotQC.R
\name{plotQC}
\alias{plotQC}
\title{Plot the QC results for specific QC steps.}
\usage{
plotQC(
  seqfile,
  QCstep = c("MissingRate", "SexCheck", "Inbreeding", "IBD", "PCA"),
  interactive = FALSE,
  sdcoef = 5,
  pc1 = "EV1",
  pc2 = "EV2",
  pairedScatter = FALSE,
  ...
)
}
\arguments{
\item{seqfile}{SeqSQC object with QC results.}

\item{QCstep}{which QC step the user want to do plotting. Takes
values of \code{c("MissingRate", "SexCheck", "Inbreeding",
"IBD", "PCA")}}

\item{interactive}{whether to generate interactive plot. Recommend
to use \code{interactive = TRUE} if user perform sample QC
using an rmarkdown script and output plot to html format.}

\item{sdcoef}{for inbreeding outlier check, how many standard
deviation we need for identification of inbreeding
outliers. The default is 5.}

\item{pc1}{the eigenvector on x axis for PCA result. The default is
"EV1" for eigenvector 1.}

\item{pc2}{the eigenvector on y axis for PCA result. The default is
"EV2" for eigenvector 2.}

\item{pairedScatter}{for PCA result, whether to plot the paired
scatterplot for the first 4 PC axes.}

\item{...}{Arguments to be passed to other methods.}
}
\value{
the ggplot or interactive plot (if output is in html
    format) for specific QC result. If "interactive=FALSE", it
    returns a ggplot and author could have the flexibility to add
    on any layers, scales, faceting specifications and coordinate
    systems.
}
\description{
Plot QC results.
}
\examples{
load(system.file("extdata", "example.seqfile.Rdata", package="SeqSQC"))
gfile <- system.file("extdata", "example.gds", package="SeqSQC")
seqfile <- SeqSQC(gdsfile = gfile, QCresult = QCresult(seqfile))
p <- plotQC(seqfile, QCstep="PCA", interactive=FALSE)
p
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LoadVfile.R
\name{LoadVfile}
\alias{LoadVfile}
\title{Data preprocessing for VCF or plink input from NGS or GWAS data.}
\usage{
LoadVfile(
  vfile,
  output = "sampleqc",
  capture.region = NULL,
  sample.annot = NULL,
  LDprune = TRUE,
  vfile.restrict = FALSE,
  slide.max.bp = 5e+05,
  ld.threshold = 0.3,
  format.data = "NGS",
  format.file = "vcf",
  ...
)
}
\arguments{
\item{vfile}{vcf or PLINK input file (ped/map/bed/bim/fam with same
basename). Vfile could be a vector of character strings, see
details.}

\item{output}{a character string for name of merged data of SeqSQC
object. The \code{dirname(output)} would be used as the
directory to save the QC results and plots. The default is
\code{sampleqc} in working directory.}

\item{capture.region}{the BED file of sequencing capture
regions. The default is NULL. For exome-sequencing data, the
capture region file must be provided.}

\item{sample.annot}{sample annotation file with 3 columns (with
header) in the order of sample id, sample population and sex
info. The default is NULL.}

\item{LDprune}{whether to use LD-pruned snp set. The default is
TRUE.}

\item{vfile.restrict}{whether the input vcf or plink file has
already been restricted by capture region. The default is
FALSE.}

\item{slide.max.bp}{the window size of SNPs when calculating
linkage disequilibrium. The default is 5e+05.}

\item{ld.threshold}{the r^2 threshold for LD-based SNP pruning if
\code{LDprune = TRUE}. The default is 0.3.}

\item{format.data}{the data source. The default is \code{NGS} for
sequencing data.}

\item{format.file}{the data format. The default is \code{vcf}.}

\item{...}{Arguments to be passed to other methods.}
}
\value{
a SeqSQC object with the filepath to the gds file which
    stores the genotype, the summary of samples and variants, and
    the QCresults including the sample annotation information.
}
\description{
Function to read VCF or plink files, merge with benchmark data, and
output as SeqSQC object.
}
\details{
For \code{vfile} with more than one file names,
    \code{LoadVfile} will merge all dataset together if they all
    contain the same samples. It is useful to combine
    genetic/genomic data together if VCF data is divided by
    chromosomes. \cr \code{sample.annot} file contains 3 columns
    with column names. col 1 is \code{sample} with sample ids; col
    2 is \code{population} with values of "AFR/EUR/ASN/EAS/SAS";
    col 3 is \code{gender} with values of "male/female".
}
\examples{
infile <- system.file("extdata", "example_sub.vcf", package="SeqSQC")
sample.annot <- system.file("extdata", "sampleAnnotation.txt", package="SeqSQC")
cr <- system.file("extdata", "CCDS.Hs37.3.reduced_chr1.bed", package="SeqSQC")
outfile <- file.path(tempdir(), "testWrapUp")
seqfile <- LoadVfile(vfile = infile, output = outfile, capture.region = cr,
sample.annot = sample.annot)
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_docum.R
\docType{data}
\name{example.gds}
\alias{example.gds}
\title{example gds file used in vignette.}
\description{
This gds file contains genotype and phenotype for 92 whole-genome
sequenced samples captured by CCDS region.  This is a merged
dataset of the 87 benchmark samples and the 5 study samples (all
are assembled from the 1000 Genomes Project).  The meta info for
these 92 samples includes sample name, pupulation, age, relation
note and group info (benchmark or study).
}
\author{
Qian Liu \email{qliu7@buffalo.edu}
}
\keyword{datasets}
